@@ -1,6 +1,7 @@
 ;; grow-pack.jl -- window resize and movement
+;; $Id: grow-pack.jl,v 1.12.2.2 2002/04/22 06:20:31 jsh Exp $
 
-;; Copyright (C) 2000, 01 Kai Grossjohann <Kai.Grossjohann@CS.Uni-Dortmund.DE>
+;; Copyright (C) 2000 Kai Grossjohann <Kai.Grossjohann@CS.Uni-Dortmund.DE>
 
 ;; This file is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -24,6 +25,13 @@
 ;; `Packing' means to move the window in the indicated direction until
 ;; it `bumps into' another window or by a given amount.
 
+;; I installed this package by placing the following into my ~/.sawfishrc:
+;;
+;; (require 'grow-pack)
+;; (require 'menus)
+;; (setq window-ops-menu
+;;       `(,@window-ops-menu ("Grow/pack" ,@grow-pack-menu)))
+
 (define-structure sawfish.wm.commands.grow-pack
 
     (export grow-window-left
@@ -46,55 +54,51 @@
 	  sawfish.wm.commands
 	  sawfish.wm.focus
 	  sawfish.wm.workspace
-	  sawfish.wm.stacking
 	  sawfish.wm.util.stacking)
 
   (define-structure-alias grow-pack sawfish.wm.commands.grow-pack)
 
 ;;; Customization options.
-
-  (defgroup gpsy "Grow, Pack, Shrink & Yank" :group move)
+   
+  (defgroup grow-pack "Growing and packing of windows" :group misc)
 
   (defcustom grow-is-maximize t
-    "Whether growing is considered to be maximization."
+    "Whether growing is considered to be maximization.  When you turn
+this on, you can use `unmaximize-window' or something similar to get
+back to the original size."
     :type boolean
-    :group (move gpsy))
+    :group (misc grow-pack))
 
   (defcustom pack-warp-pointer 'maybe
     "Whether and how to move the pointer when packing windows.
-`maybe' means that the pointer is moved along with the window, if the
-pointer was within the window area before packing. `always' warps the
-pointer to the center of the window if it isn't already in the
-window, then does like `maybe'.  `never' means not to warp the
-pointer."
-    :type (choice always maybe never)
-    :group (move gpsy))
 
-  (defcustom grow-pack-bump-obscured nil
-    "Whether to bump into fully obscured windows."
+`maybe' means that the pointer is moved along with the window, if the
+pointer was within the window area before packing.
+
+`always' warps the pointer to the center of the window if it isn't
+already in the window, then does like `maybe'.
+
+`never' means not to warp the pointer."
+    :type (choice always maybe never)
+    :group (misc grow-pack))
+
+  (defcustom grow-pack-bump-obscured ()
+    "Whether to bump into fully obscured windows when growing or packing
+windows."
     :type boolean
-    :group (move gpsy))
+    :group (misc grow-pack))
 
   (defcustom grow-pack-bump-other-depth 'always
-    "Whether to bump into windows on a different depth. When 'maybe,
-Only `avoided' windows are bumped."
+    "Whether to bump into windows on a different depth when growing or packing
+windows.
+`maybe' means only avoided windows in other depths."
     :type (choice always maybe never)
-    :group (move gpsy))
+    :group (misc grow-pack))
 
   (defcustom grow-pack-bump-ignored t
-    "Whether to bump into ignored windows."
+    "Whether to bump into ignored windows when growing or packing."
     :type boolean
-    :group (move gpsy))
-
-  (defcustom shrink-window-minimum-size 10
-    "The minimum height or width to which a window may be shrunk."
-    :type number
-    :group (move gpsy))
-
-  (defcustom yank-window-minimum-visible 10
-    "The minimum amount of window left visible, if yanked over the edge."
-    :type number
-    :group (move gpsy))
+    :group (misc grow-pack))
 
 ;;; Code:
 
@@ -157,30 +161,25 @@ See `pack-window-up'."
   (define-command 'pack-window-up pack-window-up #:spec "%W\nP")
   (define-command 'pack-window-down pack-window-down #:spec "%W\nP")
 
-  ;; Implementation part.
+  ;; Convenience variable.
 
-  (define (bump-get-head w direction)
-    (let* ((frame-dims (window-frame-dimensions w))
-           (frame-pos (window-position w))
-           (head1 (current-head w))
-           (head2 (find-head
-                   (case direction
-                     ((left) (- (car frame-pos) 1))
-                     ((right) (+ (car frame-pos) (car frame-dims)))
-                     (t (car frame-pos)))
-                   (case direction
-                     ((up) (- (cdr frame-pos) 1))
-                     ((down) (+ (cdr frame-pos) (cdr frame-dims)))
-                     (t (cdr frame-pos))))))
-      (if head2 head2 head1)))
+  (defvar grow-pack-menu
+    `((,(_ "Grow left") grow-window-left)
+      (,(_ "Grow right") grow-window-right)
+      (,(_ "Grow up") grow-window-up)
+      (,(_ "Grow down") grow-window-down)
+      (,(_ "Pack left") pack-window-left)
+      (,(_ "Pack right") pack-window-right)
+      (,(_ "Pack up") pack-window-up)
+      (,(_ "Pack down") pack-window-down))
+    "Menu of grow and pack operations.")
+
+  ;; Implementation part.
 
   (define (bump-distance w direction maximize min-dist)
     (let* ((a (window-position w))
 	   (z (window-frame-dimensions w))
 	   (depth (window-get w 'depth))
-	   (head (bump-get-head w direction))
-	   (head-dims (head-dimensions head))
-	   (head-offs (head-offset head))
 	   bump cmp
 	   xa xz
 	   min-next border xborders)
@@ -193,9 +192,7 @@ See `pack-window-up'."
 					,(y xa) . ,(+ (y xa) (y xz))))
 		  a (y a)
 		  z (+ a (y z))
-	          bump (if (eq direction 'up)
-	                   (cdr head-offs)
-	                   (car head-offs))
+		  bump 0
 		  cmp <=)
 	  (setq border (+ (x a) (x z))
 		min-next (+ border min-dist)
@@ -203,9 +200,7 @@ See `pack-window-up'."
 				      ,(y xa) . ,(+ (y xa) (y xz))))
 		a (y a)
 		z (+ a (y z))
-	        bump (if (eq direction 'down)
-	                 (+ (cdr head-offs) (cdr head-dims))
-	                 (+ (car head-offs) (car head-dims)))
+		bump ((if (eq direction 'down) screen-height screen-width))
 		cmp >=)))
       (if (cmp bump min-next)
 	  (mapc
@@ -219,14 +214,14 @@ See `pack-window-up'."
 			    (window-avoided-p x)
 			  (eq grow-pack-bump-other-depth 'always))))
 		  (or grow-pack-bump-obscured
-                      (not (eq (stacking-visibility x) 'fully-obscured)))
+		      (not (eq (window-visibility x) 'fully-obscured)))
 		  (or grow-pack-bump-ignored
 		      (not (window-ignored-p x)))
 		  (setq xa (window-position x)
 			xz (window-frame-dimensions x))
 		  (cmp bump (car (setq x (xborders))) min-next)
-		  (< a (cddr x))
-		  (< (cadr x) z)
+		  (or (< a (cadr x) z) (< a (cddr x) z)
+		      (< (cadr x) a (cddr x)) (< (cadr x) z (cddr x)))
 		  (setq bump (car x))))
 	   (managed-windows)))
       (- bump border)))
@@ -253,7 +248,7 @@ See `pack-window-up'."
 	  (window-put w 'unmaximized-geometry (list x y (car dim) (cdr dim))))
 	(window-put w
 		    (if horizontal 'maximized-horizontally
-                      'maximized-vertically)
+			'maximized-vertically)
 		    t))
       (setq dim (cons (car new-dim) (cdr new-dim)))
       (maximize-truncate-dims w new-dim (if horizontal 'horizontal 'vertical)
