@@ -1,22 +1,22 @@
 ;; move-resize.jl -- interactive moving and resizing of windows
-;; $Id: move-resize.jl,v 1.91 2002/05/29 06:39:02 jsh Exp $
+;; $Id: move-resize.jl,v 1.85.2.3 2002/05/29 07:15:34 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawfish.
+;; This file is part of sawmill.
 
-;; sawfish is free software; you can redistribute it and/or modify it
+;; sawmill is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawfish is distributed in the hope that it will be useful, but
+;; sawmill is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawfish; see the file COPYING.  If not, write to
+;; along with sawmill; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.commands.move-resize
@@ -48,50 +48,42 @@
   ;; todo:
   ;;  * obey the aspect ratio size hints
 
-
-  (defcustom move-outline-mode 'opaque
-    "How windows being moved are animated"
-    :type (choice opaque box cross elliptical draft)
-    :group (appearance animation))
-
-  (defcustom resize-outline-mode 'opaque
-    "How windows being resized are animated"
-    :type (choice opaque box cross elliptical draft)
-    :group (appearance animation))
-
-  (defcustom move-resize-raise-window nil
-    "Raise windows when they are moved or resized."
-    :type boolean
-    :group move)
-
+  (defvar move-outline-mode 'opaque
+    "How windows being moved are animated, either `opaque' or `box'")
+  
+  (defvar resize-outline-mode 'opaque
+    "How windows being resized are animated, either `opaque' or `box'")
+  
+  (defvar move-resize-raise-window nil
+    "Raise windows when they are moved or resized.")
+  
   (defcustom move-show-position nil
     "Show current position of windows while moving."
     :group move
     :type boolean)
-
+  
   (defcustom resize-show-dimensions t
     "Show current dimensions of windows while resizing."
     :group move
     :type boolean)
-
- (defcustom resize-edge-mode 'border-grab
-   "How to choose window edges when resizing."
-   :type (choice region border grab border-grab)
-   :group move)
-
+  
+  (defvar resize-edge-mode 'border-grab
+    "How to choose window edges when resizing. One of `region', `border',
+`grab', `border-grab'")
+  
   (defcustom move-snap-epsilon 12
     "Distance in pixels before window edges align with each other."
     :group move
     :type (number 0 64)
     :tooltip "When moving a window, this option lets you align one of its edges with an edge of another window.")
-
+  
   (defvar move-snap-mode 'resistance
     "How to snap together window edges, one of `magnetism', `resistance', or
 `attraction'.")
-
+  
   (defvar move-snap-ignored-windows nil
     "Snap to otherwise-ignored windows.")
-
+  
   (defvar move-resize-inhibit-configure nil
     "Only update window contents after it has stopped moving.")
 
@@ -316,10 +308,7 @@
 		 ((x-base (or (cdr (assq 'base-width move-resize-hints)) 0))
 		  (x-inc (or (cdr (assq 'width-inc move-resize-hints)) 1))
 		  (y-base (or (cdr (assq 'base-height move-resize-hints)) 0))
-		  (y-inc (or (cdr (assq 'height-inc move-resize-hints)) 1))
-		  (min-aspect (assq 'min-aspect move-resize-hints))
-		  (max-aspect (assq 'max-aspect move-resize-hints)))
-
+		  (y-inc (or (cdr (assq 'height-inc move-resize-hints)) 1)))
 	       (when (memq resize-edge-mode '(grab border-grab))
 		 (add-edges ptr-x ptr-y))
 	       (cond
@@ -328,23 +317,13 @@
 		       (constrain-dimension-to-hints
 			(+ move-resize-old-width
 			   (- ptr-x move-resize-old-ptr-x))
-			'x move-resize-hints))
-                 (when (or min-aspect max-aspect)
-                   (setq move-resize-height
-                         (constrain-aspect-to-hints
-                          move-resize-width
-                          move-resize-old-height 'x min-aspect max-aspect))))
-
+			'x move-resize-hints)))
 		((memq 'left move-resize-moving-edges)
 		 (setq move-resize-width
 		       (constrain-dimension-to-hints
 			(+ move-resize-old-width
-			   (- move-resize-old-ptr-x ptr-x)) 'x move-resize-hints))
-                 (when (or min-aspect max-aspect)
-                   (setq move-resize-height
-                         (constrain-aspect-to-hints
-                          move-resize-width
-                          move-resize-old-height 'x min-aspect max-aspect)))
+			   (- move-resize-old-ptr-x ptr-x))
+			'x move-resize-hints))
 		 (setq move-resize-x (- move-resize-old-x
 					(- move-resize-width
 					   move-resize-old-width)))))
@@ -354,24 +333,13 @@
 		       (constrain-dimension-to-hints
 			(+ move-resize-old-height
 			   (- ptr-y move-resize-old-ptr-y))
-			'y move-resize-hints))
-                 (when (or min-aspect max-aspect)
-                   (setq move-resize-width
-                         (constrain-aspect-to-hints
-                          move-resize-height
-                          move-resize-old-width 'y min-aspect max-aspect)))
-                 )
+			'y move-resize-hints)))
 		((memq 'top move-resize-moving-edges)
 		 (setq move-resize-height
 		       (constrain-dimension-to-hints
 			(+ move-resize-old-height
 			   (- move-resize-old-ptr-y ptr-y))
 			'y move-resize-hints))
-                 (when (or min-aspect max-aspect)
-                   (setq move-resize-width
-                         (constrain-aspect-to-hints
-                          move-resize-height
-                          move-resize-old-width 'y min-aspect max-aspect)))
 		 (setq move-resize-y (- move-resize-old-y
 					(- move-resize-height
 					   move-resize-old-height)))))
@@ -512,6 +480,7 @@
 		      (copy-sequence move-resize-moving-edges)
 		    (list 'top 'bottom 'left 'right)))))))
 
+
 ;;; hook functions
 
   (define (lost-window w)
@@ -521,6 +490,7 @@
   (add-hook 'unmap-notify-hook lost-window t)
   (add-hook 'destroy-notify-hook lost-window t)
 
+
 ;;; Entry points
 
   (define (move-window-interactively w)
