@@ -1,22 +1,22 @@
 ;; custom.jl -- Emacs-like ``customizing'' (but more simple)
-;; $Id: custom.jl,v 1.64 2002/11/03 10:53:59 jsh Exp $
+;; $Id: custom.jl,v 1.61.2.1 2002/04/22 06:20:29 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawfish.
+;; This file is part of sawmill.
 
-;; sawfish is free software; you can redistribute it and/or modify it
+;; sawmill is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawfish is distributed in the hope that it will be useful, but
+;; sawmill is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawfish; see the file COPYING.  If not, write to
+;; along with sawmill; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.custom
@@ -102,6 +102,7 @@
   ;; hash group names (lists of symbols) to alist of options 
   (define custom-group-table (make-table equal-hash equal))
 
+
 ;;; defining custom variables and groups
 
   (defmacro defcustom (symbol value doc #!rest keys)
@@ -221,6 +222,7 @@ Note that the value of the `:group' key is not evaluated."
 	(setq keys (nthcdr 2 keys)))
       (cons 'list (nreverse out))))
 
+
 ;;; general management
 
   (define custom-setter-table (make-table symbol-hash eq))
@@ -310,6 +312,7 @@ of choices."
 				 (string-lessp (cadr x) (cadr y))))
 			 (filter atom (cddr custom-groups))))))))
 
+
 ;;; setting values
 
   (define (custom-set setter symbol)
@@ -389,6 +392,7 @@ of choices."
 		       ;; XXX alternatives to user-eval
 		       (user-eval x))) (cdr form))))
 
+
 ;;; serializing unreadable types
 
   ;; property name (symbol) to find type converters on during custom-convert
@@ -451,21 +455,18 @@ of choices."
   (define (define-custom-deserializer type fun)
     (put type 'custom-deserializer fun))
 
+
 ;;; support for font and color primitive types
 
   (define-custom-serializer 'font (lambda (value)
 				    (if (fontp value)
-					(cons (font-type value)
-					      (font-name value))
+					(font-name value)
 				      value)))
 
   (define-custom-deserializer 'font (lambda (value)
-				      (cond ((stringp value)
-					     (get-font value))
-					    ((consp value)
-					     (get-font-typed
-					      (car value) (cdr value)))
-					    (t value))))
+				      (if (stringp value)
+					  (get-font value)
+					value)))
 
   (define-custom-serializer 'color (lambda (value)
 				     (if (colorp value)
@@ -477,17 +478,19 @@ of choices."
 					   (get-color value)
 					 value)))
 
+
 ;;; default groups
 
   (defgroup focus "Focus")
   (defgroup move "Move/Resize" :require sawfish.wm.commands.move-resize)
+  (defgroup placement "Placement")
   (defgroup appearance "Appearance")
   (defgroup workspace "Workspaces")
   (defgroup bindings "Bindings")
-  (defgroup min-max "Mini/Maximizing")
+  (defgroup min-max "Minimizing and Maximizing")
   (defgroup misc "Miscellaneous")
-  (defgroup placement "Placement" :group misc)
 
+
 ;;; loading user's customisations
 
   (define (custom-load filename)
@@ -508,6 +511,7 @@ of choices."
 	  (custom-default-file
 	   (custom-load custom-default-file))))
 
+
 ;;; init
 
   (let ((tem (get-command-line-option "--custom-file" t)))
