@@ -1,22 +1,22 @@
 ;; frames.jl -- handle window framing
-;; $Id: frames.jl,v 1.93 2003/10/21 16:17:59 jsh Exp $
+;; $Id: frames.jl,v 1.84.2.3 2002/05/22 06:55:31 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawfish.
+;; This file is part of sawmill.
 
-;; sawfish is free software; you can redistribute it and/or modify it
+;; sawmill is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawfish is distributed in the hope that it will be useful, but
+;; sawmill is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawfish; see the file COPYING.  If not, write to
+;; along with sawmill; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.frames
@@ -38,8 +38,6 @@
 	     frame-style-editable-p
 	     window-type
 	     set-window-type
-	     push-window-type
-	     pop-window-type
 	     window-type-remove-title
 	     window-type-remove-border
 	     window-type-add-title
@@ -60,7 +58,7 @@
 	  rep.io.files
 	  rep.io.timers
 	  sawfish.wm.frames.subrs
-	  sawfish.wm.windows
+	  sawfish.wm.windows.subrs
 	  sawfish.wm.misc
 	  sawfish.wm.custom
 	  sawfish.wm.commands
@@ -87,9 +85,6 @@
   ;;	unframed		no frame at all
   ;;	icon
   ;;	dock
-  ;;	menu, toolbar
-  ;;	utility
-  ;;	splash
 
   ;; There is also a similar concept of frame types. The window type
   ;; never changes (unless the user explicitly does so). But the frame
@@ -112,6 +107,7 @@
   ;; non-maximized. (In fact the frame-type != window-type scheme is
   ;; largely just for compatibility)
 
+
 ;;; custom support
 
   (define (custom-make-frame-style-widget)
@@ -119,6 +115,7 @@
 
   (put 'frame-style 'custom-widget custom-make-frame-style-widget)
 
+
 ;;; variables etc
 
   (defvar frame-part-classes nil
@@ -142,7 +139,7 @@ that overrides settings set elsewhere.")
   (defcustom reload-themes-when-changed t
     "Automatically reload themes when they are updated."
     :type boolean
-    :group appearance)
+    :group misc)
 
   (defvar frame-type-fallback-alist
     '((transient . default)
@@ -151,14 +148,7 @@ that overrides settings set elsewhere.")
       (shaded . shaped)
       (shaded-transient . shaped-transient)
       (icon . shaped-transient)
-      (dock . icon)
-      (utility . default)
-      (shaded-utility . shaded-transient)
-      (toolbar . unframed)
-;;    (shaded-toolbar . shaded-utility)
-      (menu . unframed)
-;;    (shaded-menu . shaded-utility)
-      (splash . unframed))
+      (dock . icon))
     "Frame type fallbacks.")
 
   (defvar theme-update-interval 60
@@ -205,6 +195,7 @@ deciding which frame type to ask a theme to generate.")
 
   (defvar sawfish-themer-program "sawfish-themer")
 
+
 ;;; defcustom's for some built-in variables
 
   (defcustom default-font nil
@@ -217,6 +208,7 @@ deciding which frame type to ask a theme to generate.")
   (defvar default-bevel-percent nil
     "Bevel intensity as a percentage.")
 
+
 ;;; managing frame types
 
   (define (define-frame-type-mapper fun)
@@ -243,6 +235,7 @@ deciding which frame type to ask a theme to generate.")
 	;; else, apply this transformation and keep looping
 	(loop-1 (cdr rest) ((car rest) w type)))))
 
+
 ;;; managing frame styles
 
   (define (add-frame-style name function)
@@ -287,6 +280,7 @@ deciding which frame type to ask a theme to generate.")
 		  (reload-frame-style style))))
 	    frame-style-files)))
 
+
 ;;; applying frame styles to windows
 
   (define (reframe-window w)
@@ -348,6 +342,7 @@ deciding which frame type to ask a theme to generate.")
     (when (apply-frame-style style)
       (save-current-frame-style)))
 
+
 ;;; editable frame styles
 
   (define (mark-frame-style-editable style)
@@ -365,6 +360,7 @@ deciding which frame type to ask a theme to generate.")
 	    (system (format nil "%s %s &" sawfish-themer-program dir))))))
     #:spec (lambda () (list default-frame-style)))
 
+
 ;;; kludge different window decors by modifying the assumed window type
 
   (define (window-type w)
@@ -378,34 +374,11 @@ deciding which frame type to ask a theme to generate.")
 	    'default))))
 
   (define (set-window-type w type)
-    (if (window-get w 'type/saved)
-	(window-put w 'type/saved type)
-      (unless (eq (window-get w 'type) type)
-	(window-put w 'type type)
-	(call-window-hook 'window-state-change-hook w (list '(type)))
-	(reframe-window w))))
-
-  ;; XXX do something with KEY. (It's a unique symbol used to mark
-  ;; XXX the different users of these functions)
-
-  (define (push-window-type w type key)
-    (window-put w 'type/key key)
-    (unless (eq (window-get w 'type type))
-      (unless (window-get w 'type/saved)
-	(window-put w 'type/saved (window-get w 'type)))
+    (unless (eq (window-get w 'type) type)
       (window-put w 'type type)
       (call-window-hook 'window-state-change-hook w (list '(type)))
       (reframe-window w)))
 
-  (define (pop-window-type w key)
-    (when (and (window-get w 'type/saved)
-	       (eq (window-get w 'type/key) key))
-      (window-put w 'type (window-get w 'type/saved))
-      (window-put w 'type/saved nil)
-      (window-put w 'type/key nil))
-    (call-window-hook 'window-state-change-hook w (list '(type)))
-    (reframe-window w))
-
   (define (window-type-remove-title type)
     (case type
       ((default) 'transient)
@@ -436,6 +409,7 @@ deciding which frame type to ask a theme to generate.")
 	    (lambda (w) (set-window-type w type)) #:spec "%W"))
 	'(default transient shaped shaped-transient unframed))
 
+
 ;;; loading ``themes'' (currently just frame styles)
 
   (define (frame-style-directory dir #!optional get-name)
@@ -532,6 +506,7 @@ deciding which frame type to ask a theme to generate.")
        (check . ,(eq (window-type w) 'unframed))
        (group . menu-type))))
 
+
 ;;; removing frame parts
 
   (define (remove-frame-class w class)
@@ -549,6 +524,7 @@ deciding which frame type to ask a theme to generate.")
   (define (frame-class-removed-p w class)
     (memq class (window-get w 'removed-classes)))
 
+
 ;;; manipulating the frame part classes variables
 
   (define (set-frame-part-value class key value #!optional override)
@@ -614,6 +590,7 @@ deciding which frame type to ask a theme to generate.")
 	  ((bottom-right-corner) 'bottom_right_corner))
       nil))
 
+
 ;;; initialisation
 
   (define-frame-class 'menu-button '((keymap . menu-button-keymap)))
@@ -646,13 +623,5 @@ deciding which frame type to ask a theme to generate.")
 
   (make-timer frames-on-idle theme-update-interval)
 
-  (add-hook 'sm-window-save-functions
-	    (lambda (w)
-	      (cond ((window-get w 'type/saved)
-		     (list (cons 'type (window-get w 'type/saved))))
-		    ((window-get w 'type)
-		     (list (cons 'type (window-get w 'type)))))))
-
-  (sm-add-saved-properties 'ignored 'frame-style)
-  (sm-add-restored-properties 'type)
+  (sm-add-saved-properties 'type 'ignored 'frame-style)
   (add-swapped-properties 'frame-active-color 'frame-inactive-color))
