@@ -1,22 +1,22 @@
 ;; maximize.jl -- window maximization
-;; $Id: maximize.jl,v 1.67 2002/09/23 07:33:17 jsh Exp $
+;; $Id: maximize.jl,v 1.61.2.3 2002/05/07 03:17:14 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawfish.
+;; This file is part of sawmill.
 
-;; sawfish is free software; you can redistribute it and/or modify it
+;; sawmill is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawfish is distributed in the hope that it will be useful, but
+;; sawmill is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawfish; see the file COPYING.  If not, write to
+;; along with sawmill; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.state.maximize
@@ -47,9 +47,7 @@
 	    maximize-fill-window-vertically-toggle
 	    maximize-fill-window-horizontally-toggle
 	    maximize-window-fullscreen
-	    maximize-window-fullscreen-toggle
-	    maximize-window-fullxinerama
-	    maximize-window-fullxinerama-toggle)
+	    maximize-window-fullscreen-toggle)
 
     (open rep
 	  rep.system
@@ -65,11 +63,13 @@
 	  sawfish.wm.util.display-window
 	  sawfish.wm.util.stacking
 	  sawfish.wm.frames
-	  sawfish.wm.misc
-	  sawfish.wm.focus)
+	  sawfish.wm.misc)
 
   (define-structure-alias maximize sawfish.wm.state.maximize)
 
+  ;; This sets the window property `unmaximized-geometry' of each
+  ;; currently maximize window to `(X Y W H)', the saved geometry.
+
   (defvar maximize-always-expands nil
     "Maximizing a window in one dimension must increase the size of that dimension.")
 
@@ -93,6 +93,7 @@
   ;; called when a window is un-maximized, args (W #!optional DIRECTION)
   (defvar window-unmaximized-hook nil)
 
+
 ;;; handling maximized state
 
   (define (window-maximized-p w)
@@ -113,7 +114,7 @@
       (when (window-maximized-horizontally-p w)
 	(rplaca coords (nth 0 old-geom)))
       (when (window-maximized-vertically-p w)
-	(rplacd coords (nth 1 old-geom)))
+	(rplaca coords (nth 1 old-geom)))
       coords))
 
   (define (window-unmaximized-dimensions w)
@@ -125,8 +126,6 @@
 	(rplacd dims (nth 3 old-geom)))
       dims))
 
-  ;; This sets the window property `unmaximized-geometry' of each
-  ;; currently maximize window to `(X Y W H)', the saved geometry.
   (define (save-unmaximized-geometry w)
     (unless (window-get w 'unmaximized-geometry)
       (let ((coords (window-position w))
@@ -136,7 +135,10 @@
 
   (define (discard-unmaximized-geometry w)
     (window-put w 'unmaximized-geometry nil)
-    (pop-window-type w 'sawfish.wm.state.maximize))
+    (let ((type (window-get w 'unmaximized-type)))
+      (when type
+	(set-window-type w type)
+	(window-put w 'unmaximized-type nil))))
 
   (define (unmaximized-geometry w)
     (window-get w 'unmaximized-geometry))
@@ -170,6 +172,7 @@
       (maximize-discard w (or (memq 'left edges) (memq 'right edges))
 			(or (memq 'top edges) (memq 'bottom edges)))))
 
+
 ;;; 1D packing
 
   (defmacro edges-touching (start end edge)
@@ -246,6 +249,7 @@
 			(- (cdr fdims) (cdr dims))))
 	w)))
 
+
 ;;; 2D packing
 
   (define (do-both window avoided edges coords dims fdims)
@@ -260,6 +264,7 @@
 			(- (cdr fdims) (cdr dims))))
 	window)))
 
+
 ;;; size hints stuff
 
   (define (window-maximizable-p w #!optional direction hints)
@@ -302,6 +307,7 @@
 	(rplacd dims (trunc (cdr dims) y-inc y-base y-max)))
       dims))
 
+
 ;;; misc functions
 
   (define (maximize-find-workarea #!optional w #!key head head-fallback)
@@ -329,6 +335,7 @@
 	    bottom-left-corner bottom-right-corner title)
 	   (not (and h-maximized v-maximized)))))))
 
+
 ;;; commands
 
   (define (maximize-window w #!optional direction only-1d)
@@ -400,7 +407,6 @@
 		  (not (window-maximized-horizontally-p w))
 		  (not (window-maximized-fullscreen-p w)))
 	 (discard-unmaximized-geometry w))))
-    (warp-pointer-if-necessary w)
     (call-window-hook 'window-unmaximized-hook w (list direction))
     (call-window-hook 'window-state-change-hook w (list '(maximized))))
 
@@ -441,6 +447,7 @@ unmaximized."
   (define-command 'maximize-window-horizontally-toggle maximize-window-horizontally-toggle #:spec "%W")
   (define-command 'maximize-window-vertically-toggle maximize-window-vertically-toggle #:spec "%W")
 
+
 ;;; fill commands
 
   (define (maximize-fill-window w #!optional direction)
@@ -486,16 +493,17 @@ unmaximized."
   (define-command 'maximize-fill-window-horizontally-toggle maximize-fill-window-horizontally-toggle #:spec "%W")
   (define-command 'maximize-fill-window-vertically-toggle maximize-fill-window-vertically-toggle #:spec "%W")
 
+
 ;; fullscreen commands
 
   (define (maximize-window-fullscreen w state)
-    "Fullscreen maximize the window."
     (cond ((and state (not (window-maximized-fullscreen-p w)))
 	   (when (window-maximizable-p w)
-	     (let ((head-offset (current-head-offset w)) (head-dims (current-head-dimensions w)))
+	     (let ((head-offset (current-head-offset w))
+		   (head-dims (current-head-dimensions w)))
 	       (save-unmaximized-geometry w)
 	       (window-put w 'unmaximized-type (window-type w))
-	       (push-window-type w 'unframed 'sawfish.wm.state.maximize)
+	       (set-window-type w 'unframed)
 	       (move-resize-window-to w (car head-offset) (cdr head-offset)
 				      (car head-dims) (cdr head-dims))
 	       (raise-window* w)
@@ -511,7 +519,6 @@ unmaximized."
 	   (unmaximize-window w 'fullscreen))))
 
   (define (maximize-window-fullscreen-toggle w)
-    "Toggle the state of the window between fullscreen maximized and unmaximized."
     (maximize-window-fullscreen w (not (window-maximized-fullscreen-p w))))
 
   (define-command 'maximize-window-fullscreen
@@ -519,54 +526,18 @@ unmaximized."
   (define-command 'maximize-window-fullscreen-toggle
     maximize-window-fullscreen-toggle #:spec "%W")
 
-  (define (maximize-window-fullxinerama w state)
-    "Fullscreen maximize the window across all Xinerama screens."
-    (cond ((and state (not (window-maximized-fullscreen-p w)))
-	   (when (window-maximizable-p w)
-	     ( let ((screen-dims (screen-dimensions w)))
-	       (save-unmaximized-geometry w)
-	       (window-put w 'unmaximized-type (window-type w))
-	       (push-window-type w 'unframed 'sawfish.wm.state.maximize)
-	       (move-resize-window-to w 0 0 (car screen-dims) (cdr screen-dims))
-	       (raise-window* w)
-	       (window-put w 'maximized-fullscreen t)
-	       (window-put w 'maximized-vertically t)
-	       (window-put w 'maximized-horizontally t)
-	       (call-window-hook 'window-maximized-hook
-				 w (list 'fullscreen))
-	       (call-window-hook 'window-state-change-hook
-				 w (list '(maximized))))))
-
-	  ((and (not state) (window-maximized-fullscreen-p w))
-	   (unmaximize-window w 'fullscreen))))
-
-  (define (maximize-window-fullxinerama-toggle w)
-    "Toggle the state of the window between fullscreen maximized across all Xinerama and unmaximized."
-    (maximize-window-fullxinerama w (not (window-maximized-fullscreen-p w))))
-
-  (define-command 'maximize-window-fullxinerama
-    maximize-window-fullxinerama #:spec "%W")
-  (define-command 'maximize-window-fullxinerama-toggle
-    maximize-window-fullxinerama-toggle #:spec "%W")
-
+
 ;;; initialisation
 
   (define (after-add-window w)
     (let ((vert (window-get w 'queued-vertical-maximize))
-	  (horiz (window-get w 'queued-horizontal-maximize))
-	  (full (window-get w 'queued-fullscreen-maximize)))
-      (when (or vert horiz full)
-	(when vert
-	  (window-put w 'queued-vertical-maximize nil))
-	(when horiz
-	  (window-put w 'queued-horizontal-maximize nil))
-	(when full
-	  (window-put w 'queued-fullscreen-maximize nil))
-	(if full
-	    (maximize-window-fullscreen w t)
-	  (maximize-window w (cond ((and vert horiz) nil)
-				   (vert 'vertical)
-				   (horiz 'horizontal)))))))
+	  (horiz (window-get w 'queued-horizontal-maximize)))
+      (when (or vert horiz)
+	(window-put w 'queued-vertical-maximize nil)
+	(window-put w 'queued-horizontal-maximize nil)
+	(maximize-window w (cond ((and vert horiz) nil)
+				 (vert 'vertical)
+				 (horiz 'horizontal))))))
 
   (add-hook 'after-add-window-hook after-add-window)
 
@@ -601,27 +572,9 @@ unmaximized."
 	      ;; avoided windows
 	      (add-hook 'add-window-hook check-if-maximizable)))
 
-  (add-hook 'sm-window-save-functions
-	    (lambda (w)
-	      (if (window-maximized-fullscreen-p w)
-		  (list '(maximized-fullscreen . t))
-		(nconc (and (window-maximized-horizontally-p w)
-			    (list '(maximized-horizontally . t)))
-		       (and (window-maximized-vertically-p w)
-			    (list '(maximized-vertically . t)))))))
-
-  (add-hook 'sm-restore-window-hook
-	    (lambda (w alist)
-	      (mapc (lambda (cell)
-		      (let ((v (cdr (assq (car cell) alist))))
-			(when v
-			  (window-put w (cdr cell) v))))
-		    '((maximized-fullscreen . queued-fullscreen-maximize)
-		      (maximized-vertically . queued-vertical-maximize)
-		      (maximized-horizontally . queued-horizontal-maximize)))))
-
-  (sm-add-saved-properties 'unmaximized-geometry)
-
+  (sm-add-saved-properties
+   'unmaximized-geometry 'maximized-vertically
+   'maximized-horizontally 'maximized-fullscreen)
   (add-swapped-properties
    'unmaximized-geometry 'maximized-vertically
    'maximized-horizontally 'maximized-fullscreen)
