@@ -1,21 +1,22 @@
 ;; transient.jl -- support transient windows
+;; $Id: transient.jl,v 1.59.2.2 2002/05/07 03:17:14 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawfish.
+;; This file is part of sawmill.
 
-;; sawfish is free software; you can redistribute it and/or modify it
+;; sawmill is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawfish is distributed in the hope that it will be useful, but
+;; sawmill is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawfish; see the file COPYING.  If not, write to
+;; along with sawmill; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.state.transient
@@ -36,7 +37,6 @@
 	  sawfish.wm.events
 	  sawfish.wm.custom
 	  sawfish.wm.commands
-          sawfish.wm.focus
 	  sawfish.wm.windows
 	  sawfish.wm.stacking
 	  sawfish.wm.viewport
@@ -52,6 +52,7 @@
   (defvar decorate-transients nil
     "Decorate dialog windows similarly to application windows.")
 
+
 ;;; functions
 
   (define (transient-of-p x y #!key allow-root)
@@ -70,18 +71,17 @@
 		    (eql (window-group-id x) (window-group-id y)))))))
 
   (define (indirect-transient-of-p x y #!key allow-root)
-    "Return t if window X is (directly, or indirectly) a transient for
-window Y."
+    "Return t if window X is (directly, or indirectly) a transient for window Y."
     (let loop ((x x))
-         (or (transient-of-p x y #:allow-root allow-root)
-             (let ((x-for (window-transient-p x)))
-               (and x-for
-                    ;; Some KDE windows set WM_TRANSIENT_FOR to their own id!
-                    (not (eql x-for (window-id x)))
-                    (let ((x-for-w (get-window-by-id x-for)))
-                      (if x-for-w
-                          (loop x-for-w)
-                        nil)))))))
+      (or (transient-of-p x y #:allow-root allow-root)
+	  (let ((x-for (window-transient-p x)))
+	    (and x-for
+		 ;; Some KDE windows set WM_TRANSIENT_FOR to their own id!
+		 (not (eql x-for (window-id x)))
+		 (let ((x-for-w (get-window-by-id x-for)))
+		   (if x-for-w
+		       (loop x-for-w)
+		     nil)))))))
 
   (define (transient-parents w #!optional indirectly)
     "Return the list of windows that window W is a transient for."
@@ -116,6 +116,7 @@ indirect transients of the parent window in question."
 transient group as window W."
     (mapc fun (transient-group w)))
 
+
 ;;; commands for raising windows with their transients
 
   (define (raise-window-and-transients w)
@@ -141,6 +142,7 @@ the level of any transient windows it has."
   (define-command 'raise-lower-window-and-transients
     raise-lower-window-and-transients #:spec "%W" #:class 'advanced)
 
+
 ;;; displaying
 
   (define (transient-frame-type w type)
@@ -153,31 +155,24 @@ the level of any transient windows it has."
 
   (define-frame-type-mapper transient-frame-type)
 
+
 ;;; hooks
 
-  ;; 1. Transients of the currently focused window get focus.
-  ;; 2. Transients of root belonging to the same window group as the
-  ;;    currently focused window get focus, unless the currently focused
-  ;;    window is also transient (wacky special case code in transient-of-p).
-  ;; 3. Other transients for root, transients for a desktop window
-  ;;    or non-transients may get focus depending on options (yes, we
-  ;;    also handle non-transients).
   (define (transient-map-window w)
-    (when (and (window-really-wants-input-p w)
-               (window-visible-p w)
-               (or (let ((focus (input-focus)))
-                     (and focus (transient-of-p w focus #:allow-root t)))
-                   (let ((x-for-id (window-transient-p w)))
-                     (and (or (not x-for-id)
-                              (eql x-for-id (root-window-id))
-                              (let ((x-for (get-window-by-id x-for-id)))
-                                (and x-for (window-get x-for 'desktop))))
-                          (or (and
-                               focus-windows-when-mapped
-                               (not (window-get w 'never-focus))
-                               (not (window-get w 'inhibit-focus-when-mapped)))
-                              (window-get w 'focus-when-mapped))))))
-      (set-input-focus w)))
+    (cond ((and (window-transient-p w)
+		(window-really-wants-input-p w)
+		(window-visible-p w)
+		(input-focus)
+		(transient-of-p w (input-focus) #:allow-root t))
+	   (set-input-focus w))
+	  ((and (or (and focus-windows-when-mapped
+			 (not (window-get w 'never-focus)))
+		    (window-get w 'focus-when-mapped))
+		(or (not (window-transient-p w))
+		    (eql (window-transient-p w) (root-window-id)))
+		(window-really-wants-input-p w)
+		(window-visible-p w))
+	   (set-input-focus w))))
 
   ;; If a transient window gets unmapped that currently has the input
   ;; focus, pass it (the focus) to its parent. Otherwise, pass the focus
@@ -192,15 +187,25 @@ the level of any transient windows it has."
 			 (or (window-order-most-recent
 			      #:windows (delq w (windows-in-group w)))
 			     (get-window-by-id (window-transient-p w))))))
-	(if (and parent
-                 (window-mapped-p parent)
-                 (window-visible-p parent)
-                 (not (window-outside-viewport-p parent))
-                 (window-really-wants-input-p parent)
-                 (not (window-get parent 'desktop)))
-            (set-input-focus parent)
-          ;; No parent to give focus back to.
-          (focus-revert)))))
+	(when (or (not parent)
+		  (not (window-mapped-p parent))
+		  (not (window-visible-p parent))
+		  (window-outside-viewport-p parent)
+		  (not (window-really-wants-input-p parent)))
+	  ;; if no parent, choose the topmost window (if in click-to-focus
+	  ;; mode) or the window under the pointer otherwise
+	  (if (eq focus-mode 'click)
+	      (setq parent nil)
+	    (setq parent (query-pointer-window))
+	    (when (and (eq focus-mode 'enter-only)
+		       parent (desktop-window-p parent))
+	      ;; in sloppy-focus mode, don't want to focus on the
+	      ;; desktop window just because the pointer is under it
+	      (setq parent nil)))
+	  (unless (or parent (eq focus-mode 'enter-exit))
+	    (setq parent (window-order-most-recent))))
+	(when (or (null parent) (window-really-wants-input-p parent))
+	  (set-input-focus parent)))))
 
   (add-hook 'map-notify-hook transient-map-window)
   (add-hook 'unmap-notify-hook transient-unmap-window)
