@@ -1,23 +1,23 @@
 ;; wm-spec.jl -- implement the new (GNOME/KDE) wm hints spec
 
-;; $Id: wm-spec.jl,v 1.49 2005/07/01 16:04:50 jsh Exp $
+;; $Id: wm-spec.jl,v 1.27.2.4 2002/05/29 07:16:47 jsh Exp $
 
 ;; Copyright (C) 1999, 2000 John Harper <john@dcs.warwick.ac.uk>
 
-;; This file is part of sawfish.
+;; This file is part of sawmill.
 
-;; sawfish is free software; you can redistribute it and/or modify it
+;; sawmill is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
 ;; the Free Software Foundation; either version 2, or (at your option)
 ;; any later version.
 
-;; sawfish is distributed in the hope that it will be useful, but
+;; sawmill is distributed in the hope that it will be useful, but
 ;; WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with sawfish; see the file COPYING.  If not, write to
+;; along with sawmill; see the file COPYING.  If not, write to
 ;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 
 (define-structure sawfish.wm.state.wm-spec
@@ -37,19 +37,15 @@
 
   ;; todo:
 
+  ;; - _NET_WORKAREA
+  ;; - _NET_WM_NAME		-- needs to be in C code?
   ;; - _NET_WM_ICON
 
-  ;; 1.1 additions:
-  ;;  - _NET_WM_ALLOWED_ACTIONS
-  ;;  - _STATE_HIDDEN?
-  ;;  - _NET_WM_MOVERESIZE changes
-  ;;  - _NET_SHOWING_DESKTOP?
-  ;;  - _NET_MOVERESIZE_WINDOW
-
   ;; maybe add some state extensions for things the spec doesn't
   ;; cover but existed in the old GNOME spec; e.g. _GNOME_WM_STATE_FOO
   ;; for FOO being DO_NOT_COVER, SKIP_FOCUS, ..?
 
+
 ;;; constants
 
   (defconst _NET_WM_MOVERESIZE_SIZE_TOPLEFT 0)
@@ -61,8 +57,6 @@
   (defconst _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT 6)
   (defconst _NET_WM_MOVERESIZE_SIZE_LEFT 7)
   (defconst _NET_WM_MOVERESIZE_MOVE 8)
-  (defconst _NET_WM_MOVERESIZE_SIZE_KEYBOARD 9)
-  (defconst _NET_WM_MOVERESIZE_MOVE_KEYBOARD 10)
 
   (defconst _NET_WM_STATE_REMOVE 0)
   (defconst _NET_WM_STATE_ADD 1)
@@ -81,11 +75,8 @@
      _NET_DESKTOP_VIEWPORT
      _NET_NUMBER_OF_DESKTOPS
      _NET_PROTOCOLS
-     _NET_SHOWING_DESKTOP
      _NET_SUPPORTED
      _NET_SUPPORTING_WM_CHECK
-     _NET_WORKAREA
-     _NET_WM_DESKTOP
      _NET_WM_ICON_GEOMETRY
      _NET_WM_MOVERESIZE
      _NET_WM_MOVERESIZE_MOVE
@@ -97,13 +88,9 @@
      _NET_WM_MOVERESIZE_SIZE_TOP
      _NET_WM_MOVERESIZE_SIZE_TOPLEFT
      _NET_WM_MOVERESIZE_SIZE_TOPRIGHT
-     _NET_WM_MOVERESIZE_SIZE_KEYBOARD
-     _NET_WM_MOVERESIZE_MOVE_KEYBOARD
      _NET_WM_PING
      _NET_WM_STATE
-     _NET_WM_STATE_ABOVE
      _NET_WM_STATE_ADD
-     _NET_WM_STATE_BELOW
      _NET_WM_STATE_FULLSCREEN
      _NET_WM_STATE_MAXIMIZED
      _NET_WM_STATE_MAXIMIZED_HORZ
@@ -111,25 +98,20 @@
      _NET_WM_STATE_REMOVE
      _NET_WM_STATE_SHADED
      _NET_WM_STATE_SKIP_PAGER
-     _NET_WM_STATE_SKIP_TASKBAR
      _NET_WM_STATE_STICKY
      _NET_WM_STATE_TOGGLE
      _NET_WM_STRUT
      _NET_WM_WINDOW_TYPE
      _NET_WM_WINDOW_TYPE_DESKTOP
      _NET_WM_WINDOW_TYPE_DIALOG
-     _NET_WM_WINDOW_TYPE_DOCK
-     _NET_WM_WINDOW_TYPE_TOOLBAR
-     _NET_WM_WINDOW_TYPE_MENU
-     _NET_WM_WINDOW_TYPE_UTILITY
-     _NET_WM_WINDOW_TYPE_SPLASH
-     _NET_WM_USER_TIME])
+     _NET_WM_WINDOW_TYPE_DOCK])
   
-  (defvar wm-spec-below-depth -2)
-  (defvar wm-spec-above-depth +2)
+  (defconst desktop-layer -4)
+  (defconst dock-layer +4)
 
   (define supported-states '())
 
+
 ;;; setting the client list hints
 
   (define (update-client-list-hints #!key only-stacking-list)
@@ -147,6 +129,7 @@
       (set-prop (managed-windows) '_NET_CLIENT_LIST))
     (set-prop (nreverse (stacking-order)) '_NET_CLIENT_LIST_STACKING))
 
+
 ;; setting the desktop / viewport hints
 
   (define last-workspace nil)
@@ -154,28 +137,12 @@
   (define last-workspace-names nil)
   (define last-area nil)
   (define last-area-count nil)
-  (define last-workarea nil)
-  (define last-showing-desktop nil)
-
-  (define (update-window-workspace-hints w #!key (limits (workspace-limits)))
-    (let ((vec (if (window-sticky-p/workspace w)
-		   (vector #xffffffff)
-		 (let ((space (or (window-get w 'swapped-in)
-				  (car (window-workspaces w)))))
-		   (and space (vector (- space (car limits))))))))
-      (unless (equal vec (window-get w 'wm-spec/last-workspace))
-	(if vec
-	    (set-x-property w '_NET_WM_DESKTOP vec 'CARDINAL 32)
-	  (delete-x-property w '_NET_WM_DESTOP))
-	(window-put w 'wm-spec/last-workspace vec))))
 
   (define (update-workspace-hints)
     (let* ((limits (workspace-limits))
 	   (port (screen-viewport))
 	   (port-size viewport-dimensions)
-	   (total-workspaces (1+ (- (cdr limits) (car limits))))
-	   (workarea (make-vector (* 4 total-workspaces)))
-	   (showing-desktop (showing-desktop-p)))
+	   (total-workspaces (1+ (- (cdr limits) (car limits)))))
 
       (define (set-ws-hints)
 	;; _NET_NUMBER_OF_DESKTOPS
@@ -215,31 +182,18 @@
 				  view 'CARDINAL 32)
 		(aset view (* i 2) (* (car port) (screen-width)))
 		(aset view (1+ (* i 2)) (* (cdr port) (screen-height)))
-		(loop (1+ i))))))
-
-	;; _NET_WORKAREA
-	(unless (equal last-workarea workarea)
-	  (set-x-property 'root '_NET_WORKAREA workarea 'CARDINAL 32)
-	  (setq last-workarea workarea))
-
-	;; _NET_SHOWING_DESKTOP
-	(unless (equal showing-desktop last-showing-desktop)
-	  (set-x-property 'root '_NET_SHOWING_DESKTOP
-			  (vector (if showing-desktop 1 0)) 'CARDINAL 32)
-	  (setq last-showing-desktop showing-desktop)))
+		(loop (1+ i)))))))
 
       (define (set-window-hints w)
-	(update-window-workspace-hints w #:limits limits))
-
-      ;; calculate workareas
-      (do ((i 0 (1+ i)))
-	  ((= i total-workspaces))
-	(let ((area (calculate-workarea-from-struts
-		     #:workspace (+ i (car limits)))))
-	  (aset workarea (+ (* i 4) 0) (nth 0 area))
-	  (aset workarea (+ (* i 4) 1) (nth 1 area))
-	  (aset workarea (+ (* i 4) 2) (- (nth 2 area) (nth 0 area)))
-	  (aset workarea (+ (* i 4) 3) (- (nth 3 area) (nth 1 area)))))
+	(cond ((window-sticky-p/workspace w)
+	       (set-x-property w '_NET_WM_DESKTOP
+			       (vector #xffffffff) 'CARDINAL 32))
+	      ((window-get w 'swapped-in)
+	       ;; XXX the gnome-wm standard sucks..!
+	       (let ((space (window-get w 'swapped-in)))
+		 (set-x-property w '_NET_WM_DESKTOP
+				 (vector (- space (car limits)))
+				 'CARDINAL 32)))))
 		 
       ;; apparently some pagers don't like it if we place windows
       ;; on (temporarily) non-existent workspaces
@@ -251,6 +205,7 @@
       (when (>= last-workspace-count total-workspaces)
 	(set-ws-hints))))
 
+
 ;;; setting the focus hints
 
   (define last-focus nil)
@@ -262,17 +217,21 @@
 	(set-x-property 'root '_NET_ACTIVE_WINDOW
 			(vector (if focus (window-id focus) 0)) 'WINDOW 32))))
 
+
 ;;; setting the window state hints
 
   (define (update-client-state w)
-    (let ((state '()))
+    (let ((state (filter (lambda (x) (not (supported-state-p x)))
+			 (window-get w 'wm-spec-last-states))))
       (mapc (lambda (x)
 	      (when (and (not (pseudo-state-p x))
 			 (call-state-fun w x 'get))
 		(setq state (cons x state))))
 	    supported-states)
-      (set-x-property w '_NET_WM_STATE (apply vector state) 'ATOM 32)))
+      (set-x-property w '_NET_WM_STATE (apply vector state) 'ATOM 32)
+      (window-put w 'wm-spec-last-states state)))
 
+
 ;;; honouring the initially set window state hints
 
   (define (update-icon-geometry w geom)
@@ -311,23 +270,19 @@
 	(setq state (nth 2 state))
 	(do ((i 0 (1+ i)))
 	    ((= i (length state)))
-	  (call-state-fun w (aref state i) 'init))))
+	  (call-state-fun w (aref state i) 'init))
+	(window-put w 'wm-spec-last-states (vector->list state))))
 
     (update-strut w)
 
     (let ((geom (get-x-property w '_NET_WM_ICON_GEOMETRY)))
       (when geom
-	(update-icon-geometry w (nth 2 geom))))
-      (when (equal (get-x-property w '_NET_WM_USER_TIME)
-          '(CARDINAL 32 #(0)))
-        (window-put w 'inhibit-focus-when-mapped t)))
+	(update-icon-geometry w (nth 2 geom)))))
 
+
 ;;; helper functions
 
-  (define (define-wm-spec-window-type x fun)
-    (if (listp x)
-	(mapc (lambda (y) (define-wm-spec-window-type y fun)) x)
-      (put x 'wm-spec-type fun)))
+  (define (define-wm-spec-window-type x fun) (put x 'wm-spec-type fun))
 
   (define (define-wm-spec-window-state x fun #!key pseudo)
     (put x 'wm-spec-state fun)
@@ -344,40 +299,33 @@
       (when fun
 	(fun w mode))))
 
-  (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_DESKTOP
+  (define-wm-spec-window-type
+   '_NET_WM_WINDOW_TYPE_DESKTOP
    (lambda (w)
-     (mark-window-as-desktop w)))
+     (require 'sawfish.wm.stacking)
+     (mark-window-as-desktop w)
+     (window-put w 'fixed-position t)
+     ;; I thought these would be set by the application, but KDE doesn't..
+     (window-put w 'type 'unframed)
+     (window-put w 'sticky t)
+     (window-put w 'sticky-viewport t)
+     (set-window-depth w desktop-layer)))
 
-  (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_DOCK
+  (define-wm-spec-window-type
+   '_NET_WM_WINDOW_TYPE_DOCK
    (lambda (w)
+     (require 'sawfish.wm.stacking)
+     (set-window-depth w dock-layer)
      (mark-window-as-dock w)))
 
-  (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_DIALOG
-   (lambda (w)
-     (mark-window-as-transient w)))
-
-  (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_UTILITY
+  (define-wm-spec-window-type
+   '_NET_WM_WINDOW_TYPE_DIALOG
    (lambda (w)
      (require 'sawfish.wm.frames)
-     (set-window-type w 'utility)))
+     (set-window-type w 'transient)))
 
-  (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_TOOLBAR
-   (lambda (w)
-     (require 'sawfish.wm.frames)
-     (set-window-type w 'toolbar)))
-
-  (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_MENU
-   (lambda (w)
-     (require 'sawfish.wm.frames)
-     (set-window-type w 'menu)))
-
-  (define-wm-spec-window-type '_NET_WM_WINDOW_TYPE_SPLASH
-   (lambda (w)
-     (require 'sawfish.wm.frames)
-     (set-window-type w 'splash)
-     (window-put w 'place-mode 'centered)))
-
-  (define-wm-spec-window-state '_NET_WM_STATE_STICKY
+  (define-wm-spec-window-state
+   '_NET_WM_STATE_STICKY
    (lambda (w mode)
      (case mode
        ((init)   (window-put w 'sticky-viewport t))
@@ -399,12 +347,10 @@
 	((remove) (unmaximize-window w direction))
 	((add)    (maximize-window w direction))
 	((toggle) (maximize-window-toggle w direction))
-	((get)    (if (window-maximized-fullscreen-p w)
-		      nil
-		    (case direction
-		      ((vertical) (window-maximized-vertically-p w))
-		      ((horizontal) (window-maximized-horizontally-p w))
-		      (t (window-maximized-p w))))))))
+	((get)    (case direction
+		    ((vertical) (window-maximized-vertically-p w))
+		    ((horizontal) (window-maximized-horizontally-p w))
+		    (t (window-maximized-p w)))))))
 
   (define-wm-spec-window-state '_NET_WM_STATE_MAXIMIZED_VERT
 			       (wm-spec-maximize-handler 'vertical))
@@ -414,7 +360,8 @@
 			       (wm-spec-maximize-handler nil)
 			       #:pseudo t)
 
-  (define-wm-spec-window-state '_NET_WM_STATE_SHADED
+  (define-wm-spec-window-state
+   '_NET_WM_STATE_SHADED
    (lambda (w mode)
      (require 'sawfish.wm.state.shading)
      (case mode
@@ -424,7 +371,8 @@
        ((toggle) (toggle-window-shaded w))
        ((get)    (window-get w 'shaded)))))
 
-  (define-wm-spec-window-state '_NET_WM_STATE_SKIP_PAGER
+  (define-wm-spec-window-state
+   '_NET_WM_STATE_SKIP_PAGER
    (lambda (w mode)
      (case mode
        ((init add) (window-put w 'window-list-skip t))
@@ -433,44 +381,17 @@
 			       (not (window-get w 'window-list-skip))))
        ((get)      (window-get w 'window-list-skip)))))
 
-  (define-wm-spec-window-state '_NET_WM_STATE_SKIP_TASKBAR
-   (lambda (w mode)
-     (case mode
-       ((init add) (window-put w 'task-list-skip t))
-       ((remove)   (window-put w 'task-list-skip nil))
-       ((toggle)   (window-put w 'task-list-skip
-			       (not (window-get w 'task-list-skip))))
-       ((get)      (window-get w 'task-list-skip)))))
-
-  (define-wm-spec-window-state '_NET_WM_STATE_FULLSCREEN
+  (define-wm-spec-window-state
+   '_NET_WM_STATE_FULLSCREEN
    (lambda (w mode)
      (require 'sawfish.wm.state.maximize)
      (case mode
-       ((init) (window-put w 'queued-fullscreen-maximize t))
-       ((add remove) (maximize-window-fullscreen w (eq mode 'add)))
+       ((init add) (maximize-window-fullscreen w t))
+       ((remove) (maximize-window-fullscreen w nil))
        ((toggle) (maximize-window-fullscreen-toggle w))
        ((get) (window-maximized-fullscreen-p w)))))
 
-  (define (above-below-handler depth w mode)
-    (require 'sawfish.wm.stacking)
-    (case mode
-      ((init)
-       (window-put w 'depth depth))
-      ((add remove)
-       (set-window-depth w (if (eq mode 'add) depth 0)))
-      ((toggle)
-       (set-window-depth w (if (= (window-depth w) depth) 0 depth)))
-      ((get)
-       (= (window-depth w) depth))))
-
-  (define-wm-spec-window-state '_NET_WM_STATE_BELOW
-   (lambda (w mode)
-     (above-below-handler wm-spec-below-depth w mode)))
-
-  (define-wm-spec-window-state '_NET_WM_STATE_ABOVE
-   (lambda (w mode)
-     (above-below-handler wm-spec-above-depth w mode)))
-
+
 ;;; client messages
 
   (define (client-message-handler w type data)
@@ -480,19 +401,13 @@
 	 (when (windowp w)
 	   (delete-window w)))
 
-	((_NET_SHOWING_DESKTOP)
-	 (if (= (aref data 0) 1)
-	     (show-desktop)
-	   (hide-desktop)))
-
 	((_NET_WM_MOVERESIZE)
 	 (when (and (windowp w) (window-mapped-p w))
 	   (require 'sawfish.wm.commands.move-resize)
 	   (let ((mode (aref data 2)))
 	     ;; don't want grabs failing, sigh
 	     (x-server-timestamp t t)
-	     (if (or (eq mode _NET_WM_MOVERESIZE_MOVE)
-		     (eq mode _NET_WM_MOVERESIZE_MOVE_KEYBOARD))
+	     (if (eq mode _NET_WM_MOVERESIZE_MOVE)
 		 (move-window-interactively w)
 	       (let ((move-resize-moving-edges
 		      (cond ((eq mode _NET_WM_MOVERESIZE_SIZE_TOPLEFT) '(top left))
@@ -515,12 +430,7 @@
 	 (set-viewport (aref data 0) (aref data 1)))
 
 	((_NET_CURRENT_DESKTOP)
-         ; KDE spews _NET_CURRENT_DESKTOP( -1) messages so often that it
-         ; is best to just ignore out of bounds errors silently.
-         (let ((ws (workspace-id-from-logical (aref data 0)))
-               (limits (workspace-limits)))
-           (if (<= (car limits) ws (cdr limits))
-               (select-workspace ws))))
+	 (select-workspace (workspace-id-from-logical (aref data 0))))
 
 	((_NET_DESKTOP_NAMES)
 	 ;; XXX this is kind of broken now we use workspace-names to
@@ -568,6 +478,7 @@
 	(t (setq handled nil)))
       handled))
 
+
 ;;; property changes
 
   (define (property-change-handler w prop kind)
@@ -580,6 +491,7 @@
       ((_NET_WM_STRUT)
        (update-strut w))))
 
+
 ;;; utilities
 
   (define (vector->list vec)
@@ -587,10 +499,7 @@
 	 (out '() (cons (aref vec i) out)))
 	((= i (length vec)) (nreverse out))))
 
-  (define (update-on-configure-notify w)
-    (when (eq w 'root)
-      (update-workspace-hints)))
-
+
 ;;; initialisation
 
   (define (init)
@@ -600,7 +509,7 @@
 		    (vector wm-spec-window-id) 'WINDOW 32)
     (set-x-property wm-spec-window-id '_NET_SUPPORTING_WM_CHECK
 		    (vector wm-spec-window-id) 'WINDOW 32)
-    (set-x-property wm-spec-window-id '_NET_WM_NAME "Sawfish" 'UTF8_STRING 8)
+    (set-x-property wm-spec-window-id '_NET_WM_NAME "Sawfish" 'STRING 8)
 
     (set-x-property 'root '_NET_SUPPORTED supported-atoms 'ATOM 32)
 
@@ -620,12 +529,8 @@
     (add-hook 'workspace-state-change-hook update-workspace-hints)
     (add-hook 'viewport-resized-hook update-workspace-hints)
     (add-hook 'viewport-moved-hook update-workspace-hints)
-    (add-hook 'workarea-changed-hook update-workspace-hints)
-    (add-hook 'configure-notify-hook update-on-configure-notify)
 
-    ; Better not expose work in progress.  map-notify-hook gets
-    ; called after this anyway.
-    ;(add-hook 'add-window-hook update-client-list-hints)
+    (add-hook 'add-window-hook update-client-list-hints)
     (add-hook 'destroy-notify-hook update-client-list-hints)
     (add-hook 'map-notify-hook update-client-list-hints)
     (add-hook 'unmap-notify-hook update-client-list-hints)
@@ -633,10 +538,8 @@
 
     (add-hook 'before-add-window-hook honour-client-state)
     (add-hook 'add-window-hook update-client-state)
-    (call-after-state-changed '(sticky shaded maximized stacking
-				window-list-skip task-list-skip)
+    (call-after-state-changed '(sticky shaded maximized stacking)
 			      update-client-state)
-    (call-after-state-changed 'sticky update-window-workspace-hints)
 
     (add-hook 'focus-in-hook update-focus-state)
     (add-hook 'focus-out-hook update-focus-state)
