@@ -3,6 +3,7 @@ exec rep "$0" "$@"
 !#
 
 ;; sawmill-menu -- subprocess to handle menus
+;; $Id: sawfish-menu.jl,v 1.6.2.4 2002/04/10 04:22:09 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -25,6 +26,7 @@ exec rep "$0" "$@"
 (require 'gui.gtk-2.gtk)
 (require 'rep.data.tables)
 
+
 ;; menus
 
 (define menu-selected nil)
@@ -100,14 +102,14 @@ exec rep "$0" "$@"
 
 (define (popup-menu spec #!optional timestamp position)
   (let ((menu (let-fluids ((group-table (make-group-table)))
-                          (create-menu spec))))
+		(create-menu spec))))
     (g-signal-connect menu "deactivate" gtk-main-quit)
     (setq menu-selected nil)
     (gtk-menu-popup-interp menu nil nil 0 (or timestamp 0) position)
-    (when (memq 'visible (GTK-WIDGET-FLAGS menu))
-      (gtk-main))
+    (gtk-main)
     menu-selected))
 
+
 ;; entry point, loop reading command forms, sending back results
 
 (condition-case nil
