@@ -1,31 +1,30 @@
 /* display.c -- display handling
-   $Id$
+   $Id: display.c,v 1.42.2.1 2002/04/22 06:20:41 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-   This file is part of sawfish.
+   This file is part of sawmill.
 
-   sawfish is free software; you can redistribute it and/or modify it
+   sawmill is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   sawfish is distributed in the hope that it will be useful, but
+   sawmill is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with sawfish; see the file COPYING.   If not, write to
+   along with sawmill; see the file COPYING.   If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 
-#include "sawfish.h"
+#include "sawmill.h"
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
 #include <X11/cursorfont.h>
 #include <X11/keysym.h>
 #include <X11/extensions/shape.h>
-#include <X11/Xproto.h>
 #include <string.h>
 #include <assert.h>
 #include <stdarg.h>
@@ -44,7 +43,7 @@
 # include <netdb.h>
 #endif
 
-char *visual_name;
+char *prog_name, *visual_name;
 Display *dpy;
 int screen_num, screen_width, screen_height;
 Window root_window, no_focus_window;
@@ -55,57 +54,57 @@ int preferred_depth;
 
 /* some atoms that may be useful.. */
 Atom xa_wm_state, xa_wm_change_state, xa_wm_protocols, xa_wm_delete_window,
-  xa_wm_colormap_windows, xa_wm_take_focus, xa_compound_text,
-  xa_wm_net_name, xa_wm_net_icon_name, xa_utf8_string;
+    xa_wm_colormap_windows, xa_wm_take_focus, xa_compound_text;
 
 DEFSYM(display_name, "display-name");
 DEFSYM(canonical_display_name, "canonical-display-name");
 
+
 /* X error handlers */
 
+static void
+print_error (XErrorEvent *ev)
+{
+    char buf[256];
+    XGetErrorText(dpy, ev->error_code, buf, sizeof (buf));
+    fprintf(stderr, "X Error: %s\n", buf);
+    fprintf(stderr, "  Request Major code: %d\n", ev->request_code);
+    fprintf(stderr, "  Request Minor code: %d\n", ev->minor_code);
+    fprintf(stderr, "  ResourceId 0x%x\n", (u_int)ev->resourceid);
+}
+
 /* General error handler. Probably due to lag between windows being
    killed and us receiving DestroyNotify events */
 static int
 error_handler (Display *dpy, XErrorEvent *ev)
 {
     Lisp_Window *w;
-
 #ifdef DEBUG
     print_error (ev);
 #endif
-
-    if (ev->resourceid != 0
-	&& (ev->error_code == BadWindow || ev->error_code == BadDrawable))
+    if (ev->resourceid == 0)		/* probably a deleted window */
+	return 0;
+    w = x_find_window_by_id (ev->resourceid);
+    if (w != 0 && (ev->error_code == BadWindow
+		   || ev->error_code == BadDrawable))
     {
-	w = x_find_window_by_id (ev->resourceid);
-
-	if (w != NULL)
-	{
-	    DB(("error_handler (%s)\n", rep_STR(w->name)));
-
-	    if (!WINDOW_IS_GONE_P (w))
-           {
-               /* don't unmap a window that had send an X_ConfigureWindow request */
-               if(
-                   /*     ev->type == 0 what is the "type" ? but I've seen that type is always 0 */
-                   /*&&*/ ev->error_code==BadWindow /* the window is bad, because it is not configured yet */
-                     &&   ev->request_code==X_ConfigureWindow
-                     &&   ev->minor_code==0 /* X_ConfigureWindow is not in an Xlib extension, so it must be 0 */
-               )
-               {
-                   return 0;
-               } else
-               {
-                   remove_window (w, TRUE, TRUE);
-               }
-           }
-
-	    /* so we call emit_pending_destroys () at some point */
-	    rep_mark_input_pending (ConnectionNumber (dpy)); 
-	}
+	DB(("error_handler (%s)\n", rep_STR(w->name)));
+	if (!WINDOW_IS_GONE_P (w))
+	    remove_window (w, Qt, Qt);
+	/* so we call emit_pending_destroys () at some point */
+	rep_mark_input_pending (ConnectionNumber (dpy));
+	return 0;			/* ?? */
+    }
+    else
+    {
+#if 0
+	/* I'm fed up seeing these errors! :-) */
+#ifndef DEBUG
+	print_error (ev);
+#endif
+#endif
+	return 0;
     }
-
-    return 0;
 }
 
 /* Installed whilst trying to set the root window event mask */
@@ -116,6 +115,7 @@ error_other_wm (Display *dpy, XErrorEvent *ev)
     exit (1);
 }
 
+
 
 static char *
 canonical_host (char *host)
@@ -189,6 +189,7 @@ canonical_display (char *name)
     return buf;
 }
 
+
 
 static void
 redisplay (void)
@@ -287,7 +288,7 @@ choose_visual (void)
 bool
 sys_init(char *program_name)
 {
-    char *display_name = 0, *prog_name = 0;
+    char *display_name = 0;
     repv opt;
 
 #ifdef HAVE_UNIX
@@ -336,9 +337,6 @@ sys_init(char *program_name)
 	    xa_wm_colormap_windows = XInternAtom (dpy, "WM_COLORMAP_WINDOWS", False);
 	    xa_wm_take_focus = XInternAtom (dpy, "WM_TAKE_FOCUS", False);
 	    xa_compound_text = XInternAtom (dpy, "COMPOUND_TEXT", False);
-	    xa_wm_net_name = XInternAtom (dpy, "_NET_WM_NAME", False);
-	    xa_wm_net_icon_name = XInternAtom (dpy, "_NET_WM_ICON_NAME", False);
-	    xa_utf8_string = XInternAtom (dpy, "UTF8_STRING", False);
 
 	    if (!XShapeQueryExtension (dpy, &shape_event_base,
 				       &shape_error_base))
@@ -369,7 +367,7 @@ sys_init(char *program_name)
 	    }
 
 	    /* This should _never_ be used in Real Life; only for
-	       debugging. Sawfish tries to work out when the error
+	       debugging. Sawmill tries to work out when the error
 	       handle might be called (i.e. after any XGet, XQuery, XFetch
 	       type function) and then call emit_pending_destroys ()
 	       as soon as possible, so that there's as small as possible
@@ -410,6 +408,7 @@ sys_kill (void)
     }
 }
 
+
 /* utilities */
 
 repv
@@ -456,7 +455,7 @@ void
 send_client_message (Window w, Atom a, Time time)
 {
   XClientMessageEvent ev;
-
+  
   ev.type = ClientMessage;
   ev.window = w;
   ev.message_type = xa_wm_protocols;
