@@ -1,22 +1,22 @@
 /* functions.c -- useful window manager Lisp functions
-   $Id$
+   $Id: functions.c,v 1.91.2.3 2002/05/07 03:17:17 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-   This file is part of sawfish.
+   This file is part of sawmill.
 
-   sawfish is free software; you can redistribute it and/or modify it
+   sawmill is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   sawfish is distributed in the hope that it will be useful, but
+   sawmill is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with sawfish; see the file COPYING.   If not, write to
+   along with sawmill; see the file COPYING.   If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 
 /* Define this to fake two side-by-side Xinerama style heads */
@@ -40,8 +40,7 @@
 # endif
 #endif
    
-#include "sawfish.h"
-#include <string.h>
+#include "sawmill.h"
 #include <X11/Xatom.h>
 
 /* Number of outstanding server grabs made; only when this is zero is
@@ -544,18 +543,14 @@ Return the height of the root window (in pixels).
     return rep_MAKE_INT(screen_height);
 }
 
-DEFUN("sync-server", Fsync_server, Ssync_server, (repv wait), rep_Subr0) /*
+DEFUN("sync-server", Fsync_server, Ssync_server, (void), rep_Subr0) /*
 ::doc:sawfish.wm.misc#sync-server::
-sync-server [WAIT]
+sync-server
 
-Flush all pending X requests, don't wait for them to finish, unless WAIT
-is true.
+Flush all pending X requests, don't wait for them to finish.
 ::end:: */
 {
-    if (wait == Qnil)
-	XFlush (dpy);
-    else
-	XSync (dpy, False);
+    XFlush (dpy);
     return Qt;
 }
 
@@ -575,8 +570,6 @@ WINDOW may be the symbol `root', a window object or a numeric window id.
 	return WINDOWP(win) ? atom : rep_signal_arg_error (win, 1);
     XDeleteProperty (dpy, w,
 		     XInternAtom (dpy, rep_STR(rep_SYM(atom)->name), False));
-    if (WINDOWP (win))
-	property_cache_invalidate (win, atom);
     return atom;
 }
 
@@ -641,42 +634,33 @@ symbols, representing the atoms read.
     Atom a_prop;
     Atom type;
     int format;
-    unsigned long nitems;
-    unsigned char *data = 0;
+    u_long nitems;
+    u_char *data = 0;
     repv type_sym, ret_data = Qnil;
 
     w = x_win_from_arg (win);
     rep_DECLARE2(prop, rep_SYMBOLP);
     if (w == 0)
 	return WINDOWP(win) ? Qnil : rep_signal_arg_error (win, 1);
-
-    if (WINDOWP (win))
-    {
-	ret_data = property_cache_ref (win, prop);
-	if (ret_data != rep_NULL)
-	    return ret_data;
-    }
-
     a_prop = XInternAtom (dpy, rep_STR(rep_SYM(prop)->name), False);
 
     /* First read the data.. */
     {
 	long long_length = 32;
-	unsigned long bytes_after;
+	u_long bytes_after;
 	while (1)
 	{
-	    ret_data = Qnil;
 	    if (data != 0)
 		XFree (data);
 	    if (XGetWindowProperty (dpy, w, a_prop, 0, long_length, False,
 				    AnyPropertyType, &type, &format,
 				    &nitems, &bytes_after, &data) != Success)
-		goto out;
+		return Qnil;
 	    if (type == None)
-		goto out;
+		return Qnil;
 	    if (bytes_after == 0)
 		break;
-	    long_length += (bytes_after / sizeof(unsigned long)) + 1;
+	    long_length += (bytes_after / sizeof(u_long)) + 1;
 	}
     }
 
@@ -686,45 +670,37 @@ symbols, representing the atoms read.
     /* Then convert the contents to a vector or string */
     switch (format)
     {
-	unsigned short *s_data;
-	unsigned long *l_data;
+	u_short *s_data;
+	u_long *l_data;
 	int i;
 
     case 8:
-	ret_data = rep_string_dupn ((char *) data, nitems);
+	ret_data = rep_string_dupn (data, nitems);
 	break;
 
     case 16:
 	ret_data = Fmake_vector (rep_MAKE_INT(nitems), Qnil);
-	s_data = (unsigned short *)data;
+	s_data = (u_short *)data;
 	for (i = 0; i < nitems; i++)
 	    rep_VECTI(ret_data, i) = rep_MAKE_INT(s_data[i]);
 	break;
 
     case 32:
 	ret_data = Fmake_vector (rep_MAKE_INT(nitems), Qnil);
-	l_data = (unsigned long *)data;
+	l_data = (u_long *)data;
 	for (i = 0; i < nitems; i++)
 	{
 	    repv name;
 	    if (type == XA_ATOM && (name = x_atom_symbol (l_data[i])) != Qnil)
 		rep_VECTI(ret_data, i) = name;
-	    else if (type == XA_INTEGER)
-		rep_VECTI(ret_data, i) = rep_make_long_int((long) l_data[i]);
 	    else
-		rep_VECTI(ret_data, i) = rep_make_long_uint(l_data[i] & 0xffffffffUL);
+		rep_VECTI(ret_data, i) = rep_make_long_uint(l_data[i]);
 	}
 	break;
     }
-
     XFree (data);
-    ret_data = rep_list_3 (type_sym, rep_MAKE_INT(format), ret_data);
 
-out:
-    if (WINDOWP (win))
-	property_cache_set (win, prop, ret_data, 0);
-
-    return ret_data;
+    return rep_list_3 (type_sym, rep_MAKE_INT(format), ret_data);
 }
 
 DEFUN("set-x-property", Fset_x_property, Sset_x_property,
@@ -747,8 +723,8 @@ converted to their numeric X atoms.
 {
     Window w;
     Atom a_prop, a_type;
-    unsigned long nitems;
-    unsigned char *c_data = 0;
+    u_long nitems;
+    u_char *c_data = 0;
 
     w = x_win_from_arg (win);
     rep_DECLARE2(prop, rep_SYMBOLP);
@@ -760,9 +736,6 @@ converted to their numeric X atoms.
     if (w == 0)
 	return WINDOWP(win) ? prop : rep_signal_arg_error (win, 1);
 
-    if (WINDOWP (win))
-	property_cache_set (win, prop, rep_list_3 (type, format, data), 1);
-
     /* Convert to data array */
 
     if (rep_STRINGP(data))
@@ -773,12 +746,12 @@ converted to their numeric X atoms.
     switch (rep_INT(format))
     {
 	int i;
-	unsigned short *s_data;
-	unsigned long *l_data;
+	u_short *s_data;
+	u_long *l_data;
 
     case 8:
 	if (rep_STRINGP(data))
-	    c_data = (unsigned char *) rep_STR (data);
+	    c_data = rep_STR (data);
 	else
 	{
 	    c_data = alloca (nitems);
@@ -790,8 +763,8 @@ converted to their numeric X atoms.
     case 16:
 	if (rep_STRINGP(data))
 	    return rep_signal_arg_error (data, 3);
-	c_data = alloca (nitems * sizeof (unsigned short));
-	s_data = (unsigned short *)c_data;
+	c_data = alloca (nitems * sizeof (u_short));
+	s_data = (u_short *)c_data;
 	for (i = 0; i < nitems; i++)
 	    s_data[i] = rep_INT(rep_VECTI(data, i));
 	break;
@@ -799,8 +772,8 @@ converted to their numeric X atoms.
     case 32:
 	if (rep_STRINGP(data))
 	    return rep_signal_arg_error (data, 3);
-	c_data = alloca (nitems * sizeof (unsigned long));
-	l_data = (unsigned long *)c_data;
+	c_data = alloca (nitems * sizeof (u_long));
+	l_data = (u_long *)c_data;
 	for (i = 0; i < nitems; i++)
 	{
 	    if (a_type == XA_ATOM && rep_SYMBOLP(rep_VECTI(data, i)))
@@ -817,6 +790,84 @@ converted to their numeric X atoms.
     return prop;
 }
 
+DEFUN("get-x-text-property", Fget_x_text_property, Sget_x_text_property,
+      (repv win, repv prop), rep_Subr2) /*
+::doc:sawfish.wm.misc#get-x-text-property::
+get-x-text-property WINDOW PROPERTY
+::end:: */
+{
+    Window w;
+    Atom a_prop;
+    XTextProperty t_prop;
+    repv ret = Qnil;
+
+    w = x_win_from_arg (win);
+    rep_DECLARE2(prop, rep_SYMBOLP);
+    if (w == 0)
+	return WINDOWP(win) ? Qnil : rep_signal_arg_error (win, 1);
+    a_prop = XInternAtom (dpy, rep_STR(rep_SYM(prop)->name), False);
+
+    if (XGetTextProperty (dpy, w, &t_prop, a_prop) != 0)
+    {
+	char **list;
+	int count;
+	if (XTextPropertyToStringList (&t_prop, &list, &count) != 0)
+	{
+	    int i;
+	    ret = Fmake_vector (rep_MAKE_INT(count), Qnil);
+	    for (i = 0; i < count; i++)
+		rep_VECTI(ret, i) = rep_string_dup (list[i]);
+	    XFreeStringList (list);
+	}
+	XFree (t_prop.value);
+    }
+
+    return ret;
+}
+
+DEFUN("set-x-text-property", Fset_x_text_property, Sset_x_text_property, 
+      (repv win, repv prop, repv vect, repv enc), rep_Subr4) /*
+::doc:sawfish.wm.misc#set-x-text-prooperty::
+set-x-text-property WINDOW PROPERTY STRING-VECTOR [ENCODING-ATOM]
+::end:: */
+{
+    Window w;
+    Atom a_prop, enc_atom;
+    XTextProperty t_prop;
+    char **strings;
+    int count, i;
+
+    w = x_win_from_arg (win);
+    rep_DECLARE2(prop, rep_SYMBOLP);
+    rep_DECLARE3(vect, rep_VECTORP);
+    if (w == 0)
+	return WINDOWP(win) ? Qnil : rep_signal_arg_error (win, 1);
+    a_prop = XInternAtom (dpy, rep_STR(rep_SYM(prop)->name), False);
+
+    if (rep_SYMBOLP (enc))
+	enc_atom = XInternAtom (dpy, rep_STR(rep_SYM(enc)->name), False);
+    else
+	enc_atom = 0;
+
+    count = rep_VECT_LEN(vect);
+    strings = alloca (sizeof (char *) * (count + 1));
+    for (i = 0; i < count; i++)
+    {
+	if (!rep_STRINGP(rep_VECTI(vect, i)))
+	    return rep_signal_arg_error (vect, 3);
+	strings[i] = rep_STR(rep_VECTI(vect, i));
+    }
+    if (XStringListToTextProperty (strings, count, &t_prop) != 0)
+    {
+	if (enc_atom != 0)
+	    t_prop.encoding = enc_atom;
+	XSetTextProperty (dpy, w, &t_prop, a_prop);
+	XFree (t_prop.value);
+    }
+
+    return Qt;
+}
+
 DEFUN("send-client-message", Fsend_client_message, Ssend_client_message,
       (repv win, repv type, repv data, repv format), rep_Subr4) /*
 ::doc:sawfish.wm.events#send-client-message::
@@ -936,28 +987,9 @@ Returns the numeric id of the root window of the managed screen.
     return rep_MAKE_INT (root_window);
 }
 
+
 /* xinerama support */
 
-void
-update_xinerama_info (void)
-{
-#ifdef HAVE_X11_EXTENSIONS_XINERAMA_H
-# ifndef TEST_XINERAMA
-    if (dpy != 0)
-    {
-	if (XineramaQueryExtension (dpy, &xinerama_event_base,
-				    &xinerama_error_base))
-	{
-	    xinerama_head_info = XineramaQueryScreens (dpy, &xinerama_heads);
-	}
-    }
-# else
-    xinerama_head_info = debug_heads;
-    xinerama_heads = debug_nheads;
-# endif
-#endif
-}
-
 DEFUN ("head-count", Fhead_count, Shead_count, (void), rep_Subr0)
 {
     return rep_MAKE_INT (MAX (1, xinerama_heads));
@@ -1043,6 +1075,7 @@ DEFUN ("head-offset", Fhead_offset, Shead_offset, (repv id), rep_Subr1)
     }
 }
 
+
 /* Displaying a `message' window */
 
 static Window message_win;
@@ -1062,13 +1095,13 @@ refresh_message_window ()
     if (message_win != 0)
     {
 	XGCValues values;
-	unsigned long mask;
+	u_long mask;
 	char *ptr;
 	int row = 0;
 
+	values.foreground = VCOLOR(message.fg)->pixel;
 	values.background = VCOLOR(message.bg)->pixel;
-	values.graphics_exposures = False;
-	mask = GCBackground | GCGraphicsExposures;
+	mask = GCForeground | GCBackground;
 
 	if (message.gc == 0)
 	    message.gc = XCreateGC (dpy, message_win, mask, &values);
@@ -1095,7 +1128,7 @@ refresh_message_window ()
 		    offset = (message.width - width) / 2;
 	    }
 	    x_draw_string (message_win, message.font,
-			   message.gc, VCOLOR(message.fg), offset,
+			   message.gc, offset,
 			   MSG_PAD_Y
 			   + row * (VFONT(message.font)->ascent
 				    + VFONT(message.font)->descent
@@ -1175,13 +1208,13 @@ DEFUN("display-message", Fdisplay_message, Sdisplay_message,
 	    message.fg = rep_CDR(tem);
 	    if (!COLORP(message.fg))
 	    {
-		message.fg = Fget_color (message.fg, Qnil);
+		message.fg = Fget_color (message.fg);
 		if (!message.fg)
 		    return rep_NULL;
 	    }
 	}
 	if (!COLORP(message.fg))
-	    message.fg = Fget_color (rep_VAL(&black), Qnil);
+	    message.fg = Fget_color (rep_VAL(&black));
 	if (!COLORP(message.fg))
 	    return rep_signal_arg_error (Qforeground, 1);
 
@@ -1191,13 +1224,13 @@ DEFUN("display-message", Fdisplay_message, Sdisplay_message,
 	    message.bg = rep_CDR(tem);
 	    if (!COLORP(message.bg))
 	    {
-		message.bg = Fget_color (message.bg, Qnil);
+		message.bg = Fget_color (message.bg);
 		if (!message.bg)
 		    return rep_NULL;
 	    }
 	}
 	if (!COLORP(message.bg))
-	    message.bg = Fget_color (rep_VAL(&white), Qnil);
+	    message.bg = Fget_color (rep_VAL(&white));
 	if (!COLORP(message.bg))
 	    return rep_signal_arg_error (Qbackground, 1);
 
@@ -1338,6 +1371,7 @@ DEFUN("display-message", Fdisplay_message, Sdisplay_message,
     }
 }
 
+
 /* initialisation */
 
 void
@@ -1368,6 +1402,8 @@ functions_init (void)
     rep_ADD_SUBR(Slist_x_properties);
     rep_ADD_SUBR(Sget_x_property);
     rep_ADD_SUBR(Sset_x_property);
+    rep_ADD_SUBR(Sget_x_text_property);
+    rep_ADD_SUBR(Sset_x_text_property);
     rep_ADD_SUBR(Screate_window);
     rep_ADD_SUBR(Sx_atom);
     rep_ADD_SUBR(Sx_atom_name);
@@ -1399,7 +1435,21 @@ functions_init (void)
     rep_mark_static (&message.font);
     rep_mark_static (&message.justify);
 
-    update_xinerama_info ();
+#ifdef HAVE_X11_EXTENSIONS_XINERAMA_H
+# ifndef TEST_XINERAMA
+    if (dpy != 0)
+    {
+	if (XineramaQueryExtension (dpy, &xinerama_event_base,
+				    &xinerama_error_base))
+	{
+	    xinerama_head_info = XineramaQueryScreens (dpy, &xinerama_heads);
+	}
+    }
+# else
+    xinerama_head_info = debug_heads;
+    xinerama_heads = debug_nheads;
+# endif
+#endif
 }
 
 void
