@@ -1,22 +1,22 @@
-/* main.c -- Entry point for sawfish
-   $Id$
+/* main.c -- Entry point for sawmill
+   $Id: main.c,v 1.46.2.2 2002/04/22 06:20:42 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-   This file is part of sawfish.
+   This file is part of sawmill.
 
-   sawfish is free software; you can redistribute it and/or modify it
+   sawmill is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   sawfish is distributed in the hope that it will be useful, but
+   sawmill is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with sawfish; see the file COPYING.   If not, write to
+   along with sawmill; see the file COPYING.   If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 
 /* AIX requires this to be the first thing in the file.  */
@@ -37,11 +37,10 @@
 # endif
 #endif
    
-#include "sawfish.h"
+#include "sawmill.h"
 #include "build.h"
 #include <string.h>
 #include <limits.h>
-#include <unistd.h>
 #include <X11/Xlocale.h>		/* for setlocale () */
 #ifdef HAVE_GDK_PIXBUF
 #include <glib-object.h>
@@ -50,6 +49,9 @@
 /* one of the ec_ values */
 int exit_code = ec_no_exit;
 
+/* Saved value of argv[0] */
+static char *prog_name;
+
 DEFSYM(sawfish_directory, "sawfish-directory");
 DEFSYM(sawfish_lisp_lib_directory, "sawfish-lisp-lib-directory");
 DEFSYM(sawfish_site_lisp_directory, "sawfish-site-lisp-directory");
@@ -96,7 +98,7 @@ on_idle (int since_last)
 
     if (since_last == 0 && rep_recurse_depth == 0)
     {
-	/* XXX There have been reports of sawfish locking the display,
+	/* XXX There have been reports of sawmill locking the display,
 	   XXX I've never seen it, but this may help, and shouldn't hurt.. */
 	last_event_time = get_server_timestamp ();
 	ungrab_pointer ();
@@ -334,7 +336,6 @@ main(int argc, char **argv)
     char **old_argv;
     int old_argc;
     char *lang;
-    char *prog_name;
 
     /* This will fork multiple copies, if necessary */
     multihead_init (&argc, &argv);
