@@ -1,29 +1,27 @@
 /* windows.c -- window manipulation
-   $Id$
+   $Id: windows.c,v 1.119.2.2 2002/04/22 06:20:42 jsh Exp $
 
    Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
-   This file is part of sawfish.
+   This file is part of sawmill.
 
-   sawfish is free software; you can redistribute it and/or modify it
+   sawmill is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, or (at your option)
    any later version.
 
-   sawfish is distributed in the hope that it will be useful, but
+   sawmill is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with sawfish; see the file COPYING.   If not, write to
+   along with sawmill; see the file COPYING.   If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 
-#include "sawfish.h"
+#include "sawmill.h"
 #include <assert.h>
-#include <string.h>
 #include <X11/extensions/shape.h>
-#include <glib.h>
 
 Lisp_Window *window_list;
 int window_type;
@@ -80,16 +78,15 @@ DEFSYM(south_east, "south-east");
 
 static repv gravity_map[StaticGravity+1];
 
-/* In sawfish-1.3.3, the only callback used is keymap_prop_change. */
-
-static struct prop_handler *prop_handlers;
-
 struct prop_handler {
     struct prop_handler *next;
     repv prop;
     void (*callback) (Lisp_Window *w, repv prop, repv old, repv new);
 };
 
+static struct prop_handler *prop_handlers;
+
+
 /* utilities */
 
 /* Returns true if we should manage window ID */
@@ -228,7 +225,7 @@ void
 get_window_protocols (Lisp_Window *w)
 {
     Atom *prot;
-    int n;
+    u_int n;
     w->does_wm_take_focus = 0;
     w->does_wm_delete_window = 0;
     if (XGetWMProtocols (dpy, w->id, &prot, &n) != 0)
@@ -271,6 +268,7 @@ after_local_map (Lisp_Window *w)
     Fungrab_server ();
 }
 
+
 /* manipulating the Lisp window structures */
 
 /* Return the window object containing ID, or a null pointer */
@@ -304,8 +302,6 @@ install_window_frame (Lisp_Window *w)
     DB(("install_window_frame (%s)\n", rep_STR(w->name)));
     if (!w->reparented && w->frame != 0 && !WINDOW_IS_GONE_P (w))
     {
-	XSetWindowAttributes wa;
-
 	XSelectInput (dpy, w->frame, FRAME_EVENTS);
 
 	before_local_map (w);
@@ -321,9 +317,6 @@ install_window_frame (Lisp_Window *w)
 	restack_frame_parts (w);
 	reset_frame_parts (w);
 
-	wa.win_gravity = StaticGravity;
-	XChangeWindowAttributes (dpy, w->id, CWWinGravity, &wa);
-
 	DB(("  reparented to %lx [%dx%d%+d%+d]\n",
 	    w->frame, w->frame_width, w->frame_height,
 	    w->frame_x, w->frame_y));
@@ -336,13 +329,8 @@ remove_window_frame (Lisp_Window *w)
     DB(("remove_window_frame (%s)\n", rep_STR(w->name)));
     if (w->reparented && !WINDOW_IS_GONE_P (w))
     {
-	XSetWindowAttributes wa;
-
 	/* reparent the subwindow back to the root window */
 
-	wa.win_gravity = w->attr.win_gravity;
-	XChangeWindowAttributes (dpy, w->id, CWWinGravity, &wa);
-
 	before_local_map (w);
 	XReparentWindow (dpy, w->id, root_window, w->attr.x, w->attr.y);
 	w->reparented = FALSE;
@@ -357,86 +345,20 @@ remove_window_frame (Lisp_Window *w)
     }
 }
 
-
-static repv
-text_prop_to_utf8 (XTextProperty *prop)
-{
-    repv rval = Qnil;
-    if (prop->value && prop->nitems > 0)
-    {
-        char **list;
-        int count;
-        prop->nitems = strlen((char *) prop->value);
-#ifdef X_HAVE_UTF8_STRING
-        if (Xutf8TextPropertyToTextList (dpy, prop, &list, &count) >= Success)
-        {
-            if (count > 0)
-                rval = rep_string_dup (list[0]);
-            XFreeStringList (list);
-        }
-#else
-        if (XmbTextPropertyToTextList (dpy, prop, &list, &count) >= Success)
-        {
-            if (count > 0) {
-                gchar *ustr = g_locale_to_utf8(list[0], -1, NULL, NULL, NULL);
-                if (ustr)
-                {
-                    rval = rep_string_dup (ustr);
-                    g_free (ustr);
-                }
-            }
-            XFreeStringList (list);
-        }
-#endif
-    }
-    return rval;
-}
-
-
-/* Queries X properties to get the window {icon,}name */
-static void
-get_window_name(Lisp_Window *w)
-{
-    XTextProperty prop;
-
-    /* We only try to use the utf8 properties if our xlib supports them.
-       Otherwise conversion would have to go via the current locale, which
-       might lose some characters. */
-#ifdef X_HAVE_UTF8_STRING
-    if (XGetTextProperty (dpy, w->id, &prop, xa_wm_net_name))
-        w->net_name = text_prop_to_utf8 (&prop);
-    if (XGetTextProperty (dpy, w->id, &prop, xa_wm_net_icon_name))
-        w->net_icon_name = text_prop_to_utf8 (&prop);
-#endif
-
-    if (w->net_name == Qnil && XGetWMName (dpy, w->id, &prop))
-    {
-        repv name = text_prop_to_utf8 (&prop);
-        if (name != Qnil)
-            w->name = name;
-    }
-    w->full_name = w->name;
-  
-    if (w->net_icon_name == Qnil && XGetWMIconName (dpy, w->id, &prop))
-        w->icon_name = text_prop_to_utf8 (&prop);
-    if (w->icon_name == Qnil)
-        w->icon_name = w->name;
-}
-
-
-/* Add the top-level window ID to the manager's data structures.
- * This is called only from events.c -> map_request */
+/* Add the top-level window ID to the manager's data structures */
 Lisp_Window *
 add_window (Window id)
 {
+    char *tem;
     Lisp_Window *w = rep_ALLOC_CELL(sizeof (Lisp_Window));
     if (w != 0)
     {
 	rep_GC_root gc_win;
 	repv win = rep_VAL(w);
 	XWindowChanges xwc;
-	unsigned int xwcm;
+	u_int xwcm;
 	long supplied;
+	XTextProperty prop;
 
 	DB(("add_window (%lx)\n", id));
 
@@ -455,13 +377,6 @@ add_window (Window id)
 	w->plist = Qnil;
 	w->frame_style = Qnil;;
 	w->icon_image = rep_NULL;
-	w->name = rep_null_string ();
-	w->net_name = Qnil;
-	w->net_icon_name = Qnil;
-
-        /* Don't garbage collect the window before we are done. */
-        /* Note: must not return without rep_POPGC. */
-	rep_PUSHGC(gc_win, win);
 
 	/* have to put it somewhere until it finds the right place */
 	insert_in_stacking_list_above_all (w);
@@ -474,12 +389,40 @@ add_window (Window id)
 	DB(("  orig: width=%d height=%d x=%d y=%d\n",
 	    w->attr.width, w->attr.height, w->attr.x, w->attr.y));
 
-	get_window_name(w);
+	if (XGetWMName (dpy, id, &prop) && prop.value)
+	{
+	    if (prop.nitems > 0)
+	    {
+		char **list;
+		int count;
+		prop.nitems = strlen(prop.value);
+		if (XmbTextPropertyToTextList (dpy, &prop, &list, &count)
+		    >= Success)
+		{
+		    if (count > 0)
+			w->name = rep_string_dup (list[0]);
+		    XFreeStringList (list);
+		}
+	    }
+	    XFree (prop.value);
+	}
+	if (w->name == 0)
+	    w->name = rep_null_string ();
+	w->full_name = w->name;
+	if (XGetIconName (dpy, id, &tem))
+	{
+	    w->icon_name = rep_string_dup (tem);
+	    XFree (tem);
+	}
+	else
+	    w->icon_name = w->name;
 
 	w->wmhints = XGetWMHints (dpy, id);
 	if (!XGetWMNormalHints (dpy, w->id, &w->hints, &supplied))
 	    w->hints.flags = 0;
 	get_window_protocols (w);
+	if (!XGetTransientForHint (dpy, w->id, &w->transient_for_hint))
+	    w->transient_for_hint = 0;
 	if (!XGetWMColormapWindows (dpy, w->id,
 				    &w->cmap_windows, &w->n_cmap_windows))
 	{
@@ -489,7 +432,7 @@ add_window (Window id)
 	{
 	    /* Is the window shaped? */
 	    int xws, yws, xbs, ybs;
-	    unsigned int wws, hws, wbs, hbs;
+	    u_int wws, hws, wbs, hbs;
 	    int bounding, clip;
 	    XShapeSelectInput (dpy, w->id, ShapeNotifyMask);
 	    XShapeQueryExtents (dpy, w->id, &bounding, &xws, &yws, &wws, &hws,
@@ -515,31 +458,11 @@ add_window (Window id)
 	if (initialising)
 	    Fwindow_put (rep_VAL (w), Qplaced, Qt);
 
-	/* If the window requires to start as icon, then iconify it.
-	 * It is better to be done before 'before_add_window_hook', where
-	 * matching takes place, because in future, matcher can have an
-	 * option to un-iconify, overrinding the application's request
-	 * to iconify. 
-	 */
-	if (w->wmhints && w->wmhints->flags & StateHint
-	    && w->wmhints->initial_state == IconicState)
-	  {
-	    DEFSTRING (iconify_mod, "sawfish.wm.state.iconify");
-	    rep_call_lisp1 (module_symbol_value
-			    (rep_VAL (&iconify_mod), Qiconify_window),
-			    rep_VAL(w));
-	  }
-	
-	/* Prevent hook call on non existing window */
-	if (WINDOW_IS_GONE_P (w))
-	{
-		rep_POPGC;
-		return 0;
-	}
-	
 	/* ..then call the add-window-hook's.. */
+	rep_PUSHGC(gc_win, win);
 	Fcall_window_hook (Qbefore_add_window_hook, rep_VAL(w), Qnil, Qnil);
 	Fcall_window_hook (Qadd_window_hook, rep_VAL(w), Qnil, Qnil);
+	rep_POPGC;
 
 	/* In case the window disappeared during the hook call */
 	if (!WINDOW_IS_GONE_P (w))
@@ -560,11 +483,13 @@ add_window (Window id)
 
 	if (!WINDOW_IS_GONE_P (w))
 	{
-            repv tem = Fwindow_get (rep_VAL(w), Qplaced, Qnil);
+	    repv tem = Fwindow_get (rep_VAL(w), Qplaced);
 	    if (initialising || (tem && tem == Qnil))
 	    {
 		/* ..then the place-window-hook.. */
+		rep_PUSHGC(gc_win, win);
 		Fcall_window_hook (Qplace_window_hook, rep_VAL(w), Qnil, Qor);
+		rep_POPGC;
 	    }
 	}
 	Fwindow_put (rep_VAL(w), Qplaced, Qt);
@@ -577,23 +502,21 @@ add_window (Window id)
 	    /* Tell the window where it ended up.. */
 	    send_synthetic_configure (w);
 	}
-        rep_POPGC;
     }
     return w;
 }
 
-/* Remove W from the managed windows. If DESTROYED is nil and
-   the window is currently reparented by us, it will be reparented back to
-   the root window */
+/* Remove W from the managed windows. If DESTROYED is nil, then the
+   window will be reparented back to the root window */
 void
-remove_window (Lisp_Window *w, bool destroyed, bool from_error)
+remove_window (Lisp_Window *w, repv destroyed, repv from_error)
 {
     DB(("remove_window (%s, %s)\n",
-	rep_STR(w->name), destroyed ? "destroyed" : "not-destroyed"));
+	rep_STR(w->name), destroyed == Qnil ? "nil" : "t"));
 
     if (w->id != 0)
     {
-	if (!destroyed && !from_error)
+	if (destroyed == Qnil && from_error == Qnil)
 	{
 	    grab_window_events (w, FALSE);
 	    remove_window_frame (w);
@@ -602,21 +525,20 @@ remove_window (Lisp_Window *w, bool destroyed, bool from_error)
 	    XSetWindowBorderWidth (dpy, w->id, w->attr.border_width);
 	}
 
-	if (!from_error)
+	if (from_error == Qnil)
 	    destroy_window_frame (w, FALSE);
 
 	if (!WINDOW_IS_GONE_P (w))
 	    remove_from_stacking_list (w);
 
-	if (!from_error)
-	    focus_off_window (w);
+	focus_off_window (w);
 
 	w->id = 0;
 	pending_destroys++;
 
 	/* gc will do the rest... */
     }
-    else if (w->frame != 0 && !from_error)
+    else if (w->frame != 0 && from_error == Qnil)
 	destroy_window_frame (w, FALSE);
 }
 
@@ -658,22 +580,17 @@ emit_pending_destroys (void)
     pending_destroys = 0;
 }
 
+
 /* Lisp functions */
 
 DEFUN("window-get", Fwindow_get, Swindow_get,
-      (repv win, repv prop, repv checker), rep_Subr3) /*
+      (repv win, repv prop), rep_Subr2) /*
 ::doc:sawfish.wm.windows.subrs#window-get::
-window-get WINDOW PROPERTY &optional CHECKER
+window-get WINDOW PROPERTY
 
 Return the value of the property named PROPERTY (a symbol) of WINDOW.
 
 Note that these are Lisp properties not X properties.
-
-If the optional argument CHECKER is nil, then the return value is
-nil, either when the property value is nil, or the property is absent.
-
-If CHECKER is non-nil, than it returns CHECKER if the property
-is unset.
 ::end:: */
 {
     repv plist;
@@ -689,7 +606,7 @@ is unset.
 	}
 	plist = rep_CDR(rep_CDR(plist));
     }
-    return checker;
+    return Qnil;
 }
 
 DEFUN("map-window-properties", Fmap_window_properties,
@@ -755,10 +672,10 @@ Note that these are Lisp properties not X properties.
 		if (ph->prop == prop && old != val)
 		    ph->callback (VWIN (win), prop, old, val);
 	    }
-	    rep_CADR(plist) = val;
+	    rep_CAR(rep_CDR(plist)) = val;
 	    return val;
 	}
-	plist = rep_CDDR(plist);
+	plist = rep_CDR(rep_CDR(plist));
     }
     plist = Fcons(prop, Fcons(val, VWIN(win)->plist));
     if (plist != rep_NULL)
@@ -766,54 +683,6 @@ Note that these are Lisp properties not X properties.
     return val;
 }
 
-DEFUN("window-remprop", Fwindow_remprop, Swindow_remprop,
-      (repv win, repv prop), rep_Subr2) /*
-::doc:sawfish.wm.windows.subrs#window-prop-del::
-window-put WINDOW PROPERTY
-
-Delete PROPERTY of WINDOW. Return t for success, nil if WINDOW
-did not have PROPERTY.
-::end:: */
-{
-    repv *pplist;
-    rep_DECLARE1(win, XWINDOWP);
-    pplist = &VWIN(win)->plist;
-    while (rep_CONSP(*pplist) && rep_CONSP(rep_CDR(*pplist)))
-    {
-	if (rep_CAR(*pplist) == prop
-	    || (!rep_SYMBOLP(prop)
-		&& rep_value_cmp (rep_CAR(*pplist), prop) == 0))
-	{
-	    struct prop_handler *ph;
-            repv old = rep_CADR(*pplist);
-            if (old != Qnil)
-                for (ph = prop_handlers; ph != 0; ph = ph->next)
-                    if (ph->prop == prop)
-                        ph->callback(VWIN (win), prop, old, Qnil);
-            *pplist = rep_CDDR(*pplist);
-	    return Qt;
-	}
-	pplist = &rep_CDDR(*pplist);
-    }
-    return Qnil;
-}
-
-DEFUN("window-plist", Fwindow_plist, Swindow_plist,
-      (repv win), rep_Subr1) /*
-::doc:sawfish.wm.windows.subrs#window-plist::
-window-plist WINDOW
-
-Returns the property list of the window window which is of the form
-(prop value prop value ...).
-
-Do not attempt to change properties by modifying the property list in place.
-Use window-put instead.
-::end:: */
-{
-    rep_DECLARE1(win, XWINDOWP);
-    return VWIN(win)->plist;
-}
-
 DEFUN("window-name", Fwindow_name, Swindow_name, (repv win), rep_Subr1) /*
 ::doc:sawfish.wm.windows.subrs#window-name::
 window-name WINDOW
@@ -821,10 +690,8 @@ window-name WINDOW
 Return the name of window object WINDOW.
 ::end:: */
 {
-    Lisp_Window * w;
     rep_DECLARE1(win, WINDOWP);
-    w = VWIN(win);
-    return w->net_name != Qnil ? w->net_name : w->name;
+    return VWIN(win)->name;
 }
 
 DEFUN("window-full-name", Fwindow_full_name, Swindow_full_name,
@@ -847,10 +714,8 @@ window-icon-name WINDOW
 Return the name of window object WINDOW's icon.
 ::end:: */
 {
-    Lisp_Window * w;
     rep_DECLARE1(win, WINDOWP);
-    w = VWIN(win);
-    return w->net_icon_name != Qnil ? w->net_icon_name : w->icon_name;
+    return VWIN(win)->icon_name;
 }
 
 DEFUN("window-mapped-p", Fwindow_mapped_p, Swindow_mapped_p,
@@ -1108,6 +973,20 @@ symbols are `fully-obscured', `partially-obscured' or `unobscured'.
     return sym;
 }
 
+DEFUN("window-transient-p", Fwindow_transient_p, Swindow_transient_p,
+      (repv win), rep_Subr1) /*
+::doc:sawfish.wm.windows.subrs#window-transient-p::
+window-transient-p WINDOW
+
+Return non-nil if WINDOW is a transient window. The returned value will
+then be the numeric id of its parent window.
+::end:: */
+{
+    rep_DECLARE1(win, WINDOWP);
+    return (VWIN(win)->transient_for_hint
+	    ? rep_MAKE_INT(VWIN(win)->transient_for_hint) : Qnil);
+}
+
 DEFUN("window-urgent-p", Fwindow_urgent_p, Swindow_urgent_p,
       (repv win), rep_Subr1) /*
 ::doc:sawfish.wm.windows.subrs#window-urgent-p::
@@ -1213,19 +1092,6 @@ Return t if WINDOW has been reparented to a frame window.
     return VWIN(win)->reparented ? Qt : Qnil;
 }
 
-DEFUN("window-frame-id", Fwindow_frame_id, Swindow_frame_id,
-      (repv win), rep_Subr1) /*
-::doc:sawfish.wm.windows.subrs#window-frame-id::
-window-frame-id WINDOW
-
-Return the numeric id of the framing window associated with object
-WINDOW. Returns nil if the client window has no frame.
-::end:: */
-{
-    rep_DECLARE1(win, WINDOWP);
-    return VWIN(win)->frame ? rep_MAKE_INT (VWIN(win)->frame) : Qnil;
-}
-
 DEFUN("window-id", Fwindow_id, Swindow_id, (repv win), rep_Subr1) /*
 ::doc:sawfish.wm.windows.subrs#window-id::
 window-id WINDOW
@@ -1288,7 +1154,7 @@ associated with WINDOW. Possible keys in the alist are `min-height',
 
     /* Some sanity checking */
     if ((flags & PMinSize) 
-	&& (hints->min_width < 0 || hints->min_height < 0))
+	&& (hints->min_width <= 0 || hints->min_height <= 0))
 	flags &= ~PMinSize;
     if ((flags & PMaxSize)
 	&& (hints->max_width <= 0 || hints->max_height <= 0))
@@ -1302,8 +1168,9 @@ associated with WINDOW. Possible keys in the alist are `min-height',
 
     if (flags & PMinSize)
     {
-	ret = Fcons (Fcons (Qmin_width, rep_MAKE_INT(MAX(hints->min_width,1))),
-		     Fcons (Fcons (Qmin_height, rep_MAKE_INT(MAX(hints->min_height, 1))), ret));
+	ret = Fcons (Fcons (Qmin_width, rep_MAKE_INT(hints->min_width)),
+		     Fcons (Fcons (Qmin_height,
+				   rep_MAKE_INT(hints->min_height)), ret));
     }
     if (flags & PMaxSize)
     {
@@ -1368,7 +1235,7 @@ description of HOOK-TYPE.
     rep_PUSHGC(gc_hook, hook);
     rep_PUSHGC(gc_args, args);
     rep_PUSHGC(gc_type, type);
-    tem = Fwindow_get (win, hook, Qnil);
+    tem = Fwindow_get (win, hook);
     if (tem && tem != Qnil)
     {
 	tem = Fcall_hook (tem, args, type);
@@ -1418,31 +1285,27 @@ WINDOW. Returns the symbol `nil' if no such image.
        {
 	   Atom actual_type;
 	   int actual_format;
-	   unsigned long nitems, bytes_after;
-	   union {
-	       unsigned long *l;
-	       unsigned char *c;
-	   } data;
+	   long nitems, bytes_after;
+	   u_long *data = 0;
 
 	   static Atom kwm_win_icon = 0;
 
 	   if (kwm_win_icon == 0)
 	       kwm_win_icon = XInternAtom (dpy, "KWM_WIN_ICON", False);
 
-	   data.l = 0;
 	   if (XGetWindowProperty (dpy, VWIN (win)->id, kwm_win_icon,
 				   0, 2, False, kwm_win_icon,
 				   &actual_type, &actual_format,
 				   &nitems, &bytes_after,
-				   &data.c) == Success
+				   (u_char **) &data) == Success
 	       && actual_type == kwm_win_icon
 	       && bytes_after == 0)
 	   {
-	       pixmap_id = data.l[0];
-	       mask_id = data.l[1];
+	       pixmap_id = data[0];
+	       mask_id = data[1];
 	   }
-	   if (data.l != 0)
-	       XFree (data.l);
+	   if (data != 0)
+	       XFree (data);
        }
 
        VWIN (win)->icon_image = Qnil;
@@ -1519,6 +1382,7 @@ Return the list of windows that match the predicate function PRED.
     return output;
 }
 
+
 /* type hooks */
 
 static int
@@ -1544,8 +1408,6 @@ window_mark (repv win)
     rep_MARKVAL(VWIN(win)->name);
     rep_MARKVAL(VWIN(win)->full_name);
     rep_MARKVAL(VWIN(win)->icon_name);
-    rep_MARKVAL(VWIN(win)->net_name);
-    rep_MARKVAL(VWIN(win)->net_icon_name);
     rep_MARKVAL(VWIN(win)->icon_image);
 }
 
@@ -1590,6 +1452,7 @@ window_sweep (void)
     }
 }
 
+
 /* initialisation */
 
 void
@@ -1665,8 +1528,6 @@ windows_init (void)
     rep_ADD_SUBR(Swindow_get);
     rep_ADD_SUBR(Smap_window_properties);
     rep_ADD_SUBR(Swindow_put);
-    rep_ADD_SUBR(Swindow_remprop);
-    rep_ADD_SUBR(Swindow_plist);
     rep_ADD_SUBR(Swindow_name);
     rep_ADD_SUBR(Swindow_full_name);
     rep_ADD_SUBR(Swindow_icon_name);
@@ -1686,13 +1547,13 @@ windows_init (void)
     rep_ADD_SUBR(Sget_window_by_id);
     rep_ADD_SUBR(Sstacking_order);
     rep_ADD_SUBR(Swindow_visibility);
+    rep_ADD_SUBR(Swindow_transient_p);
     rep_ADD_SUBR(Swindow_urgent_p);
     rep_ADD_SUBR(Swindow_shaped_p);
     rep_ADD_SUBR(Shide_window);
     rep_ADD_SUBR(Sshow_window);
     rep_ADD_SUBR(Swindow_visible_p);
     rep_ADD_SUBR(Swindow_framed_p);
-    rep_ADD_SUBR(Swindow_frame_id);
     rep_ADD_SUBR(Swindow_id);
     rep_ADD_SUBR(Swindow_group_id);
     rep_ADD_SUBR(Swindow_size_hints);
@@ -1767,7 +1628,7 @@ windows_kill (void)
     {
 	next = rep_VAL (w->next);
 	Fcall_window_hook (Qremove_window_hook, rep_VAL (w), Qnil, Qnil);
-	remove_window (w, FALSE, FALSE);
+	remove_window (w, Qnil, Qnil);
 	w = VWIN (next);
     }
 }
